(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three);
    global.DragControls = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.DragControls = void 0;

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var _plane = new _three.Plane();

  var _raycaster = new _three.Raycaster();

  var _pointer = new _three.Vector2();

  var _offset = new _three.Vector3();

  var _intersection = new _three.Vector3();

  var _worldPosition = new _three.Vector3();

  var _inverseMatrix = new _three.Matrix4();

  var DragControls = /*#__PURE__*/function (_EventDispatcher) {
    _inherits(DragControls, _EventDispatcher);

    var _super = _createSuper(DragControls);

    function DragControls(_objects, _camera, _domElement) {
      var _this;

      _classCallCheck(this, DragControls);

      _this = _super.call(this);
      _domElement.style.touchAction = 'none'; // disable touch scroll

      var _selected = null,
          _hovered = null;
      var _intersections = []; //

      var scope = _assertThisInitialized(_this);

      function activate() {
        _domElement.addEventListener('pointermove', onPointerMove);

        _domElement.addEventListener('pointerdown', onPointerDown);

        _domElement.addEventListener('pointerup', onPointerCancel);

        _domElement.addEventListener('pointerleave', onPointerCancel);
      }

      function deactivate() {
        _domElement.removeEventListener('pointermove', onPointerMove);

        _domElement.removeEventListener('pointerdown', onPointerDown);

        _domElement.removeEventListener('pointerup', onPointerCancel);

        _domElement.removeEventListener('pointerleave', onPointerCancel);

        _domElement.style.cursor = '';
      }

      function dispose() {
        deactivate();
      }

      function getObjects() {
        return _objects;
      }

      function getRaycaster() {
        return _raycaster;
      }

      function onPointerMove(event) {
        if (scope.enabled === false) return;
        updatePointer(event);

        _raycaster.setFromCamera(_pointer, _camera);

        if (_selected) {
          if (_raycaster.ray.intersectPlane(_plane, _intersection)) {
            _selected.position.copy(_intersection.sub(_offset).applyMatrix4(_inverseMatrix));
          }

          scope.dispatchEvent({
            type: 'drag',
            object: _selected
          });
          return;
        } // hover support


        if (event.pointerType === 'mouse' || event.pointerType === 'pen') {
          _intersections.length = 0;

          _raycaster.setFromCamera(_pointer, _camera);

          _raycaster.intersectObjects(_objects, true, _intersections);

          if (_intersections.length > 0) {
            var object = _intersections[0].object;

            _plane.setFromNormalAndCoplanarPoint(_camera.getWorldDirection(_plane.normal), _worldPosition.setFromMatrixPosition(object.matrixWorld));

            if (_hovered !== object && _hovered !== null) {
              scope.dispatchEvent({
                type: 'hoveroff',
                object: _hovered
              });
              _domElement.style.cursor = 'auto';
              _hovered = null;
            }

            if (_hovered !== object) {
              scope.dispatchEvent({
                type: 'hoveron',
                object: object
              });
              _domElement.style.cursor = 'pointer';
              _hovered = object;
            }
          } else {
            if (_hovered !== null) {
              scope.dispatchEvent({
                type: 'hoveroff',
                object: _hovered
              });
              _domElement.style.cursor = 'auto';
              _hovered = null;
            }
          }
        }
      }

      function onPointerDown(event) {
        if (scope.enabled === false) return;
        updatePointer(event);
        _intersections.length = 0;

        _raycaster.setFromCamera(_pointer, _camera);

        _raycaster.intersectObjects(_objects, true, _intersections);

        if (_intersections.length > 0) {
          _selected = scope.transformGroup === true ? _objects[0] : _intersections[0].object;

          _plane.setFromNormalAndCoplanarPoint(_camera.getWorldDirection(_plane.normal), _worldPosition.setFromMatrixPosition(_selected.matrixWorld));

          if (_raycaster.ray.intersectPlane(_plane, _intersection)) {
            _inverseMatrix.copy(_selected.parent.matrixWorld).invert();

            _offset.copy(_intersection).sub(_worldPosition.setFromMatrixPosition(_selected.matrixWorld));
          }

          _domElement.style.cursor = 'move';
          scope.dispatchEvent({
            type: 'dragstart',
            object: _selected
          });
        }
      }

      function onPointerCancel() {
        if (scope.enabled === false) return;

        if (_selected) {
          scope.dispatchEvent({
            type: 'dragend',
            object: _selected
          });
          _selected = null;
        }

        _domElement.style.cursor = _hovered ? 'pointer' : 'auto';
      }

      function updatePointer(event) {
        var rect = _domElement.getBoundingClientRect();

        _pointer.x = (event.clientX - rect.left) / rect.width * 2 - 1;
        _pointer.y = -(event.clientY - rect.top) / rect.height * 2 + 1;
      }

      activate(); // API

      _this.enabled = true;
      _this.transformGroup = false;
      _this.activate = activate;
      _this.deactivate = deactivate;
      _this.dispose = dispose;
      _this.getObjects = getObjects;
      _this.getRaycaster = getRaycaster;
      return _this;
    }

    return _createClass(DragControls);
  }(_three.EventDispatcher);

  _exports.DragControls = DragControls;
});
