(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three);
    global.PointerLockControls = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.PointerLockControls = void 0;

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var _euler = new _three.Euler(0, 0, 0, 'YXZ');

  var _vector = new _three.Vector3();

  var _changeEvent = {
    type: 'change'
  };
  var _lockEvent = {
    type: 'lock'
  };
  var _unlockEvent = {
    type: 'unlock'
  };

  var _PI_2 = Math.PI / 2;

  var PointerLockControls = /*#__PURE__*/function (_EventDispatcher) {
    _inherits(PointerLockControls, _EventDispatcher);

    var _super = _createSuper(PointerLockControls);

    function PointerLockControls(camera, domElement) {
      var _this;

      _classCallCheck(this, PointerLockControls);

      _this = _super.call(this);

      if (domElement === undefined) {
        console.warn('THREE.PointerLockControls: The second parameter "domElement" is now mandatory.');
        domElement = document.body;
      }

      _this.domElement = domElement;
      _this.isLocked = false; // Set to constrain the pitch of the camera
      // Range is 0 to Math.PI radians

      _this.minPolarAngle = 0; // radians

      _this.maxPolarAngle = Math.PI; // radians

      var scope = _assertThisInitialized(_this);

      function onMouseMove(event) {
        if (scope.isLocked === false) return;
        var movementX = event.movementX || event.mozMovementX || event.webkitMovementX || 0;
        var movementY = event.movementY || event.mozMovementY || event.webkitMovementY || 0;

        _euler.setFromQuaternion(camera.quaternion);

        _euler.y -= movementX * 0.002;
        _euler.x -= movementY * 0.002;
        _euler.x = Math.max(_PI_2 - scope.maxPolarAngle, Math.min(_PI_2 - scope.minPolarAngle, _euler.x));
        camera.quaternion.setFromEuler(_euler);
        scope.dispatchEvent(_changeEvent);
      }

      function onPointerlockChange() {
        if (scope.domElement.ownerDocument.pointerLockElement === scope.domElement) {
          scope.dispatchEvent(_lockEvent);
          scope.isLocked = true;
        } else {
          scope.dispatchEvent(_unlockEvent);
          scope.isLocked = false;
        }
      }

      function onPointerlockError() {
        console.error('THREE.PointerLockControls: Unable to use Pointer Lock API');
      }

      _this.connect = function () {
        scope.domElement.ownerDocument.addEventListener('mousemove', onMouseMove);
        scope.domElement.ownerDocument.addEventListener('pointerlockchange', onPointerlockChange);
        scope.domElement.ownerDocument.addEventListener('pointerlockerror', onPointerlockError);
      };

      _this.disconnect = function () {
        scope.domElement.ownerDocument.removeEventListener('mousemove', onMouseMove);
        scope.domElement.ownerDocument.removeEventListener('pointerlockchange', onPointerlockChange);
        scope.domElement.ownerDocument.removeEventListener('pointerlockerror', onPointerlockError);
      };

      _this.dispose = function () {
        this.disconnect();
      };

      _this.getObject = function () {
        // retaining this method for backward compatibility
        return camera;
      };

      _this.getDirection = function () {
        var direction = new _three.Vector3(0, 0, -1);
        return function (v) {
          return v.copy(direction).applyQuaternion(camera.quaternion);
        };
      }();

      _this.moveForward = function (distance) {
        // move forward parallel to the xz-plane
        // assumes camera.up is y-up
        _vector.setFromMatrixColumn(camera.matrix, 0);

        _vector.crossVectors(camera.up, _vector);

        camera.position.addScaledVector(_vector, distance);
      };

      _this.moveRight = function (distance) {
        _vector.setFromMatrixColumn(camera.matrix, 0);

        camera.position.addScaledVector(_vector, distance);
      };

      _this.lock = function () {
        this.domElement.requestPointerLock();
      };

      _this.unlock = function () {
        scope.domElement.ownerDocument.exitPointerLock();
      };

      _this.connect();

      return _this;
    }

    return _createClass(PointerLockControls);
  }(_three.EventDispatcher);

  _exports.PointerLockControls = PointerLockControls;
});
