(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three);
    global.CurveExtras = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.Curves = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  /**
   * A bunch of parametric curves
   *
   * Formulas collected from various sources
   * http://mathworld.wolfram.com/HeartCurve.html
   * http://mathdl.maa.org/images/upload_library/23/stemkoski/knots/page6.html
   * http://en.wikipedia.org/wiki/Viviani%27s_curve
   * http://mathdl.maa.org/images/upload_library/23/stemkoski/knots/page4.html
   * http://www.mi.sanu.ac.rs/vismath/taylorapril2011/Taylor.pdf
   * https://prideout.net/blog/old/blog/index.html@p=44.html
   */
  // GrannyKnot
  var GrannyKnot = /*#__PURE__*/function (_Curve) {
    _inherits(GrannyKnot, _Curve);

    var _super = _createSuper(GrannyKnot);

    function GrannyKnot() {
      _classCallCheck(this, GrannyKnot);

      return _super.apply(this, arguments);
    }

    _createClass(GrannyKnot, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        t = 2 * Math.PI * t;
        var x = -0.22 * Math.cos(t) - 1.28 * Math.sin(t) - 0.44 * Math.cos(3 * t) - 0.78 * Math.sin(3 * t);
        var y = -0.1 * Math.cos(2 * t) - 0.27 * Math.sin(2 * t) + 0.38 * Math.cos(4 * t) + 0.46 * Math.sin(4 * t);
        var z = 0.7 * Math.cos(3 * t) - 0.4 * Math.sin(3 * t);
        return point.set(x, y, z).multiplyScalar(20);
      }
    }]);

    return GrannyKnot;
  }(_three.Curve); // HeartCurve


  var HeartCurve = /*#__PURE__*/function (_Curve2) {
    _inherits(HeartCurve, _Curve2);

    var _super2 = _createSuper(HeartCurve);

    function HeartCurve() {
      var _this;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 5;

      _classCallCheck(this, HeartCurve);

      _this = _super2.call(this);
      _this.scale = scale;
      return _this;
    }

    _createClass(HeartCurve, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        t *= 2 * Math.PI;
        var x = 16 * Math.pow(Math.sin(t), 3);
        var y = 13 * Math.cos(t) - 5 * Math.cos(2 * t) - 2 * Math.cos(3 * t) - Math.cos(4 * t);
        var z = 0;
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return HeartCurve;
  }(_three.Curve); // Viviani's Curve


  var VivianiCurve = /*#__PURE__*/function (_Curve3) {
    _inherits(VivianiCurve, _Curve3);

    var _super3 = _createSuper(VivianiCurve);

    function VivianiCurve() {
      var _this2;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 70;

      _classCallCheck(this, VivianiCurve);

      _this2 = _super3.call(this);
      _this2.scale = scale;
      return _this2;
    }

    _createClass(VivianiCurve, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        t = t * 4 * Math.PI; // normalized to 0..1

        var a = this.scale / 2;
        var x = a * (1 + Math.cos(t));
        var y = a * Math.sin(t);
        var z = 2 * a * Math.sin(t / 2);
        return point.set(x, y, z);
      }
    }]);

    return VivianiCurve;
  }(_three.Curve); // KnotCurve


  var KnotCurve = /*#__PURE__*/function (_Curve4) {
    _inherits(KnotCurve, _Curve4);

    var _super4 = _createSuper(KnotCurve);

    function KnotCurve() {
      _classCallCheck(this, KnotCurve);

      return _super4.apply(this, arguments);
    }

    _createClass(KnotCurve, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        t *= 2 * Math.PI;
        var R = 10;
        var s = 50;
        var x = s * Math.sin(t);
        var y = Math.cos(t) * (R + s * Math.cos(t));
        var z = Math.sin(t) * (R + s * Math.cos(t));
        return point.set(x, y, z);
      }
    }]);

    return KnotCurve;
  }(_three.Curve); // HelixCurve


  var HelixCurve = /*#__PURE__*/function (_Curve5) {
    _inherits(HelixCurve, _Curve5);

    var _super5 = _createSuper(HelixCurve);

    function HelixCurve() {
      _classCallCheck(this, HelixCurve);

      return _super5.apply(this, arguments);
    }

    _createClass(HelixCurve, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        var a = 30; // radius

        var b = 150; // height

        var t2 = 2 * Math.PI * t * b / 30;
        var x = Math.cos(t2) * a;
        var y = Math.sin(t2) * a;
        var z = b * t;
        return point.set(x, y, z);
      }
    }]);

    return HelixCurve;
  }(_three.Curve); // TrefoilKnot


  var TrefoilKnot = /*#__PURE__*/function (_Curve6) {
    _inherits(TrefoilKnot, _Curve6);

    var _super6 = _createSuper(TrefoilKnot);

    function TrefoilKnot() {
      var _this3;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;

      _classCallCheck(this, TrefoilKnot);

      _this3 = _super6.call(this);
      _this3.scale = scale;
      return _this3;
    }

    _createClass(TrefoilKnot, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        t *= Math.PI * 2;
        var x = (2 + Math.cos(3 * t)) * Math.cos(2 * t);
        var y = (2 + Math.cos(3 * t)) * Math.sin(2 * t);
        var z = Math.sin(3 * t);
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return TrefoilKnot;
  }(_three.Curve); // TorusKnot


  var TorusKnot = /*#__PURE__*/function (_Curve7) {
    _inherits(TorusKnot, _Curve7);

    var _super7 = _createSuper(TorusKnot);

    function TorusKnot() {
      var _this4;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;

      _classCallCheck(this, TorusKnot);

      _this4 = _super7.call(this);
      _this4.scale = scale;
      return _this4;
    }

    _createClass(TorusKnot, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        var p = 3;
        var q = 4;
        t *= Math.PI * 2;
        var x = (2 + Math.cos(q * t)) * Math.cos(p * t);
        var y = (2 + Math.cos(q * t)) * Math.sin(p * t);
        var z = Math.sin(q * t);
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return TorusKnot;
  }(_three.Curve); // CinquefoilKnot


  var CinquefoilKnot = /*#__PURE__*/function (_Curve8) {
    _inherits(CinquefoilKnot, _Curve8);

    var _super8 = _createSuper(CinquefoilKnot);

    function CinquefoilKnot() {
      var _this5;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;

      _classCallCheck(this, CinquefoilKnot);

      _this5 = _super8.call(this);
      _this5.scale = scale;
      return _this5;
    }

    _createClass(CinquefoilKnot, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        var p = 2;
        var q = 5;
        t *= Math.PI * 2;
        var x = (2 + Math.cos(q * t)) * Math.cos(p * t);
        var y = (2 + Math.cos(q * t)) * Math.sin(p * t);
        var z = Math.sin(q * t);
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return CinquefoilKnot;
  }(_three.Curve); // TrefoilPolynomialKnot


  var TrefoilPolynomialKnot = /*#__PURE__*/function (_Curve9) {
    _inherits(TrefoilPolynomialKnot, _Curve9);

    var _super9 = _createSuper(TrefoilPolynomialKnot);

    function TrefoilPolynomialKnot() {
      var _this6;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;

      _classCallCheck(this, TrefoilPolynomialKnot);

      _this6 = _super9.call(this);
      _this6.scale = scale;
      return _this6;
    }

    _createClass(TrefoilPolynomialKnot, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        t = t * 4 - 2;
        var x = Math.pow(t, 3) - 3 * t;
        var y = Math.pow(t, 4) - 4 * t * t;
        var z = 1 / 5 * Math.pow(t, 5) - 2 * t;
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return TrefoilPolynomialKnot;
  }(_three.Curve);

  function scaleTo(x, y, t) {
    var r = y - x;
    return t * r + x;
  } // FigureEightPolynomialKnot


  var FigureEightPolynomialKnot = /*#__PURE__*/function (_Curve10) {
    _inherits(FigureEightPolynomialKnot, _Curve10);

    var _super10 = _createSuper(FigureEightPolynomialKnot);

    function FigureEightPolynomialKnot() {
      var _this7;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;

      _classCallCheck(this, FigureEightPolynomialKnot);

      _this7 = _super10.call(this);
      _this7.scale = scale;
      return _this7;
    }

    _createClass(FigureEightPolynomialKnot, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        t = scaleTo(-4, 4, t);
        var x = 2 / 5 * t * (t * t - 7) * (t * t - 10);
        var y = Math.pow(t, 4) - 13 * t * t;
        var z = 1 / 10 * t * (t * t - 4) * (t * t - 9) * (t * t - 12);
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return FigureEightPolynomialKnot;
  }(_three.Curve); // DecoratedTorusKnot4a


  var DecoratedTorusKnot4a = /*#__PURE__*/function (_Curve11) {
    _inherits(DecoratedTorusKnot4a, _Curve11);

    var _super11 = _createSuper(DecoratedTorusKnot4a);

    function DecoratedTorusKnot4a() {
      var _this8;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 40;

      _classCallCheck(this, DecoratedTorusKnot4a);

      _this8 = _super11.call(this);
      _this8.scale = scale;
      return _this8;
    }

    _createClass(DecoratedTorusKnot4a, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        t *= Math.PI * 2;
        var x = Math.cos(2 * t) * (1 + 0.6 * (Math.cos(5 * t) + 0.75 * Math.cos(10 * t)));
        var y = Math.sin(2 * t) * (1 + 0.6 * (Math.cos(5 * t) + 0.75 * Math.cos(10 * t)));
        var z = 0.35 * Math.sin(5 * t);
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return DecoratedTorusKnot4a;
  }(_three.Curve); // DecoratedTorusKnot4b


  var DecoratedTorusKnot4b = /*#__PURE__*/function (_Curve12) {
    _inherits(DecoratedTorusKnot4b, _Curve12);

    var _super12 = _createSuper(DecoratedTorusKnot4b);

    function DecoratedTorusKnot4b() {
      var _this9;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 40;

      _classCallCheck(this, DecoratedTorusKnot4b);

      _this9 = _super12.call(this);
      _this9.scale = scale;
      return _this9;
    }

    _createClass(DecoratedTorusKnot4b, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        var fi = t * Math.PI * 2;
        var x = Math.cos(2 * fi) * (1 + 0.45 * Math.cos(3 * fi) + 0.4 * Math.cos(9 * fi));
        var y = Math.sin(2 * fi) * (1 + 0.45 * Math.cos(3 * fi) + 0.4 * Math.cos(9 * fi));
        var z = 0.2 * Math.sin(9 * fi);
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return DecoratedTorusKnot4b;
  }(_three.Curve); // DecoratedTorusKnot5a


  var DecoratedTorusKnot5a = /*#__PURE__*/function (_Curve13) {
    _inherits(DecoratedTorusKnot5a, _Curve13);

    var _super13 = _createSuper(DecoratedTorusKnot5a);

    function DecoratedTorusKnot5a() {
      var _this10;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 40;

      _classCallCheck(this, DecoratedTorusKnot5a);

      _this10 = _super13.call(this);
      _this10.scale = scale;
      return _this10;
    }

    _createClass(DecoratedTorusKnot5a, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        var fi = t * Math.PI * 2;
        var x = Math.cos(3 * fi) * (1 + 0.3 * Math.cos(5 * fi) + 0.5 * Math.cos(10 * fi));
        var y = Math.sin(3 * fi) * (1 + 0.3 * Math.cos(5 * fi) + 0.5 * Math.cos(10 * fi));
        var z = 0.2 * Math.sin(20 * fi);
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return DecoratedTorusKnot5a;
  }(_three.Curve); // DecoratedTorusKnot5c


  var DecoratedTorusKnot5c = /*#__PURE__*/function (_Curve14) {
    _inherits(DecoratedTorusKnot5c, _Curve14);

    var _super14 = _createSuper(DecoratedTorusKnot5c);

    function DecoratedTorusKnot5c() {
      var _this11;

      var scale = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 40;

      _classCallCheck(this, DecoratedTorusKnot5c);

      _this11 = _super14.call(this);
      _this11.scale = scale;
      return _this11;
    }

    _createClass(DecoratedTorusKnot5c, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        var fi = t * Math.PI * 2;
        var x = Math.cos(4 * fi) * (1 + 0.5 * (Math.cos(5 * fi) + 0.4 * Math.cos(20 * fi)));
        var y = Math.sin(4 * fi) * (1 + 0.5 * (Math.cos(5 * fi) + 0.4 * Math.cos(20 * fi)));
        var z = 0.35 * Math.sin(15 * fi);
        return point.set(x, y, z).multiplyScalar(this.scale);
      }
    }]);

    return DecoratedTorusKnot5c;
  }(_three.Curve);

  var Curves = {
    GrannyKnot: GrannyKnot,
    HeartCurve: HeartCurve,
    VivianiCurve: VivianiCurve,
    KnotCurve: KnotCurve,
    HelixCurve: HelixCurve,
    TrefoilKnot: TrefoilKnot,
    TorusKnot: TorusKnot,
    CinquefoilKnot: CinquefoilKnot,
    TrefoilPolynomialKnot: TrefoilPolynomialKnot,
    FigureEightPolynomialKnot: FigureEightPolynomialKnot,
    DecoratedTorusKnot4a: DecoratedTorusKnot4a,
    DecoratedTorusKnot4b: DecoratedTorusKnot4b,
    DecoratedTorusKnot5a: DecoratedTorusKnot5a,
    DecoratedTorusKnot5c: DecoratedTorusKnot5c
  };
  _exports.Curves = Curves;
});
