(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "../curves/NURBSUtils.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("../curves/NURBSUtils.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.NURBSUtils);
    global.NURBSCurve = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, NURBSUtils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.NURBSCurve = void 0;
  NURBSUtils = _interopRequireWildcard(NURBSUtils);

  function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

  function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  /**
   * NURBS curve object
   *
   * Derives from Curve, overriding getPoint and getTangent.
   *
   * Implementation is based on (x, y [, z=0 [, w=1]]) control points with w=weight.
   *
   **/
  var NURBSCurve = /*#__PURE__*/function (_Curve) {
    _inherits(NURBSCurve, _Curve);

    var _super = _createSuper(NURBSCurve);

    function NURBSCurve(degree, knots
    /* array of reals */
    , controlPoints
    /* array of Vector(2|3|4) */
    , startKnot
    /* index in knots */
    , endKnot
    /* index in knots */
    ) {
      var _this;

      _classCallCheck(this, NURBSCurve);

      _this = _super.call(this);
      _this.degree = degree;
      _this.knots = knots;
      _this.controlPoints = []; // Used by periodic NURBS to remove hidden spans

      _this.startKnot = startKnot || 0;
      _this.endKnot = endKnot || _this.knots.length - 1;

      for (var i = 0; i < controlPoints.length; ++i) {
        // ensure Vector4 for control points
        var point = controlPoints[i];
        _this.controlPoints[i] = new _three.Vector4(point.x, point.y, point.z, point.w);
      }

      return _this;
    }

    _createClass(NURBSCurve, [{
      key: "getPoint",
      value: function getPoint(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var point = optionalTarget;
        var u = this.knots[this.startKnot] + t * (this.knots[this.endKnot] - this.knots[this.startKnot]); // linear mapping t->u
        // following results in (wx, wy, wz, w) homogeneous point

        var hpoint = NURBSUtils.calcBSplinePoint(this.degree, this.knots, this.controlPoints, u);

        if (hpoint.w !== 1.0) {
          // project to 3D space: (wx, wy, wz, w) -> (x, y, z, 1)
          hpoint.divideScalar(hpoint.w);
        }

        return point.set(hpoint.x, hpoint.y, hpoint.z);
      }
    }, {
      key: "getTangent",
      value: function getTangent(t) {
        var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
        var tangent = optionalTarget;
        var u = this.knots[0] + t * (this.knots[this.knots.length - 1] - this.knots[0]);
        var ders = NURBSUtils.calcNURBSDerivatives(this.degree, this.knots, this.controlPoints, u, 1);
        tangent.copy(ders[1]).normalize();
        return tangent;
      }
    }]);

    return NURBSCurve;
  }(_three.Curve);

  _exports.NURBSCurve = NURBSCurve;
});
