(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "../curves/NURBSUtils.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("../curves/NURBSUtils.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.NURBSUtils);
    global.NURBSSurface = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, NURBSUtils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.NURBSSurface = void 0;
  NURBSUtils = _interopRequireWildcard(NURBSUtils);

  function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

  function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  /**
   * NURBS surface object
   *
   * Implementation is based on (x, y [, z=0 [, w=1]]) control points with w=weight.
   **/
  var NURBSSurface = /*#__PURE__*/function () {
    function NURBSSurface(degree1, degree2, knots1, knots2
    /* arrays of reals */
    , controlPoints
    /* array^2 of Vector(2|3|4) */
    ) {
      _classCallCheck(this, NURBSSurface);

      this.degree1 = degree1;
      this.degree2 = degree2;
      this.knots1 = knots1;
      this.knots2 = knots2;
      this.controlPoints = [];
      var len1 = knots1.length - degree1 - 1;
      var len2 = knots2.length - degree2 - 1; // ensure Vector4 for control points

      for (var i = 0; i < len1; ++i) {
        this.controlPoints[i] = [];

        for (var j = 0; j < len2; ++j) {
          var point = controlPoints[i][j];
          this.controlPoints[i][j] = new _three.Vector4(point.x, point.y, point.z, point.w);
        }
      }
    }

    _createClass(NURBSSurface, [{
      key: "getPoint",
      value: function getPoint(t1, t2, target) {
        var u = this.knots1[0] + t1 * (this.knots1[this.knots1.length - 1] - this.knots1[0]); // linear mapping t1->u

        var v = this.knots2[0] + t2 * (this.knots2[this.knots2.length - 1] - this.knots2[0]); // linear mapping t2->u

        NURBSUtils.calcSurfacePoint(this.degree1, this.degree2, this.knots1, this.knots2, this.controlPoints, u, v, target);
      }
    }]);

    return NURBSSurface;
  }();

  _exports.NURBSSurface = NURBSSurface;
});
