(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "./ParametricGeometry.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("./ParametricGeometry.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.ParametricGeometry);
    global.ParametricGeometries = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, _ParametricGeometry4) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.ParametricGeometries = void 0;

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  /**
   * Experimenting of primitive geometry creation using Surface Parametric equations
   */
  var ParametricGeometries = {
    klein: function klein(v, u, target) {
      u *= Math.PI;
      v *= 2 * Math.PI;
      u = u * 2;
      var x, z;

      if (u < Math.PI) {
        x = 3 * Math.cos(u) * (1 + Math.sin(u)) + 2 * (1 - Math.cos(u) / 2) * Math.cos(u) * Math.cos(v);
        z = -8 * Math.sin(u) - 2 * (1 - Math.cos(u) / 2) * Math.sin(u) * Math.cos(v);
      } else {
        x = 3 * Math.cos(u) * (1 + Math.sin(u)) + 2 * (1 - Math.cos(u) / 2) * Math.cos(v + Math.PI);
        z = -8 * Math.sin(u);
      }

      var y = -2 * (1 - Math.cos(u) / 2) * Math.sin(v);
      target.set(x, y, z);
    },
    plane: function plane(width, height) {
      return function (u, v, target) {
        var x = u * width;
        var y = 0;
        var z = v * height;
        target.set(x, y, z);
      };
    },
    mobius: function mobius(u, t, target) {
      // flat mobius strip
      // http://www.wolframalpha.com/input/?i=M%C3%B6bius+strip+parametric+equations&lk=1&a=ClashPrefs_*Surface.MoebiusStrip.SurfaceProperty.ParametricEquations-
      u = u - 0.5;
      var v = 2 * Math.PI * t;
      var a = 2;
      var x = Math.cos(v) * (a + u * Math.cos(v / 2));
      var y = Math.sin(v) * (a + u * Math.cos(v / 2));
      var z = u * Math.sin(v / 2);
      target.set(x, y, z);
    },
    mobius3d: function mobius3d(u, t, target) {
      // volumetric mobius strip
      u *= Math.PI;
      t *= 2 * Math.PI;
      u = u * 2;
      var phi = u / 2;
      var major = 2.25,
          a = 0.125,
          b = 0.65;
      var x = a * Math.cos(t) * Math.cos(phi) - b * Math.sin(t) * Math.sin(phi);
      var z = a * Math.cos(t) * Math.sin(phi) + b * Math.sin(t) * Math.cos(phi);
      var y = (major + x) * Math.sin(u);
      x = (major + x) * Math.cos(u);
      target.set(x, y, z);
    }
  };
  /*********************************************
   *
   * Parametric Replacement for TubeGeometry
   *
   *********************************************/

  _exports.ParametricGeometries = ParametricGeometries;

  ParametricGeometries.TubeGeometry = /*#__PURE__*/function (_ParametricGeometry) {
    _inherits(TubeGeometry, _ParametricGeometry);

    var _super = _createSuper(TubeGeometry);

    function TubeGeometry(path) {
      var _this;

      var segments = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 64;
      var radius = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
      var segmentsRadius = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 8;
      var closed = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

      _classCallCheck(this, TubeGeometry);

      var numpoints = segments + 1;
      var frames = path.computeFrenetFrames(segments, closed),
          tangents = frames.tangents,
          normals = frames.normals,
          binormals = frames.binormals;
      var position = new _three.Vector3();

      function ParametricTube(u, v, target) {
        v *= 2 * Math.PI;
        var i = Math.floor(u * (numpoints - 1));
        path.getPointAt(u, position);
        var normal = normals[i];
        var binormal = binormals[i];
        var cx = -radius * Math.cos(v); // TODO: Hack: Negating it so it faces outside.

        var cy = radius * Math.sin(v);
        position.x += cx * normal.x + cy * binormal.x;
        position.y += cx * normal.y + cy * binormal.y;
        position.z += cx * normal.z + cy * binormal.z;
        target.copy(position);
      }

      _this = _super.call(this, ParametricTube, segments, segmentsRadius); // proxy internals

      _this.tangents = tangents;
      _this.normals = normals;
      _this.binormals = binormals;
      _this.path = path;
      _this.segments = segments;
      _this.radius = radius;
      _this.segmentsRadius = segmentsRadius;
      _this.closed = closed;
      return _this;
    }

    return _createClass(TubeGeometry);
  }(_ParametricGeometry4.ParametricGeometry);
  /*********************************************
    *
    * Parametric Replacement for TorusKnotGeometry
    *
    *********************************************/


  ParametricGeometries.TorusKnotGeometry = /*#__PURE__*/function (_ParametricGeometries) {
    _inherits(TorusKnotGeometry, _ParametricGeometries);

    var _super2 = _createSuper(TorusKnotGeometry);

    function TorusKnotGeometry() {
      var _this2;

      var radius = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 200;
      var tube = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 40;
      var segmentsT = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 64;
      var segmentsR = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 8;
      var p = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 2;
      var q = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 3;

      _classCallCheck(this, TorusKnotGeometry);

      var TorusKnotCurve = /*#__PURE__*/function (_Curve) {
        _inherits(TorusKnotCurve, _Curve);

        var _super3 = _createSuper(TorusKnotCurve);

        function TorusKnotCurve() {
          _classCallCheck(this, TorusKnotCurve);

          return _super3.apply(this, arguments);
        }

        _createClass(TorusKnotCurve, [{
          key: "getPoint",
          value: function getPoint(t) {
            var optionalTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _three.Vector3();
            var point = optionalTarget;
            t *= Math.PI * 2;
            var r = 0.5;
            var x = (1 + r * Math.cos(q * t)) * Math.cos(p * t);
            var y = (1 + r * Math.cos(q * t)) * Math.sin(p * t);
            var z = r * Math.sin(q * t);
            return point.set(x, y, z).multiplyScalar(radius);
          }
        }]);

        return TorusKnotCurve;
      }(_three.Curve);

      var segments = segmentsT;
      var radiusSegments = segmentsR;
      var extrudePath = new TorusKnotCurve();
      _this2 = _super2.call(this, extrudePath, segments, tube, radiusSegments, true, false);
      _this2.radius = radius;
      _this2.tube = tube;
      _this2.segmentsT = segmentsT;
      _this2.segmentsR = segmentsR;
      _this2.p = p;
      _this2.q = q;
      return _this2;
    }

    return _createClass(TorusKnotGeometry);
  }(ParametricGeometries.TubeGeometry);
  /*********************************************
    *
    * Parametric Replacement for SphereGeometry
    *
    *********************************************/


  ParametricGeometries.SphereGeometry = /*#__PURE__*/function (_ParametricGeometry2) {
    _inherits(SphereGeometry, _ParametricGeometry2);

    var _super4 = _createSuper(SphereGeometry);

    function SphereGeometry(size, u, v) {
      _classCallCheck(this, SphereGeometry);

      function sphere(u, v, target) {
        u *= Math.PI;
        v *= 2 * Math.PI;
        var x = size * Math.sin(u) * Math.cos(v);
        var y = size * Math.sin(u) * Math.sin(v);
        var z = size * Math.cos(u);
        target.set(x, y, z);
      }

      return _super4.call(this, sphere, u, v);
    }

    return _createClass(SphereGeometry);
  }(_ParametricGeometry4.ParametricGeometry);
  /*********************************************
    *
    * Parametric Replacement for PlaneGeometry
    *
    *********************************************/


  ParametricGeometries.PlaneGeometry = /*#__PURE__*/function (_ParametricGeometry3) {
    _inherits(PlaneGeometry, _ParametricGeometry3);

    var _super5 = _createSuper(PlaneGeometry);

    function PlaneGeometry(width, depth, segmentsWidth, segmentsDepth) {
      _classCallCheck(this, PlaneGeometry);

      function plane(u, v, target) {
        var x = u * width;
        var y = 0;
        var z = v * depth;
        target.set(x, y, z);
      }

      return _super5.call(this, plane, segmentsWidth, segmentsDepth);
    }

    return _createClass(PlaneGeometry);
  }(_ParametricGeometry4.ParametricGeometry);
});
