(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three);
    global.InteractiveGroup = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.InteractiveGroup = void 0;

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var _pointer = new _three.Vector2();

  var _event = {
    type: '',
    data: _pointer
  };

  var InteractiveGroup = /*#__PURE__*/function (_Group) {
    _inherits(InteractiveGroup, _Group);

    var _super = _createSuper(InteractiveGroup);

    function InteractiveGroup(renderer, camera) {
      var _this;

      _classCallCheck(this, InteractiveGroup);

      _this = _super.call(this);

      var scope = _assertThisInitialized(_this);

      var raycaster = new _three.Raycaster();
      var tempMatrix = new _three.Matrix4(); // Pointer Events

      var element = renderer.domElement;

      function onPointerEvent(event) {
        event.stopPropagation();
        _pointer.x = event.clientX / element.clientWidth * 2 - 1;
        _pointer.y = -(event.clientY / element.clientHeight) * 2 + 1;
        raycaster.setFromCamera(_pointer, camera);
        var intersects = raycaster.intersectObjects(scope.children, false);

        if (intersects.length > 0) {
          var intersection = intersects[0];
          var object = intersection.object;
          var uv = intersection.uv;
          _event.type = event.type;

          _event.data.set(uv.x, 1 - uv.y);

          object.dispatchEvent(_event);
        }
      }

      element.addEventListener('pointerdown', onPointerEvent);
      element.addEventListener('pointerup', onPointerEvent);
      element.addEventListener('pointermove', onPointerEvent);
      element.addEventListener('mousedown', onPointerEvent);
      element.addEventListener('mouseup', onPointerEvent);
      element.addEventListener('mousemove', onPointerEvent);
      element.addEventListener('click', onPointerEvent); // WebXR Controller Events
      // TODO: Dispatch pointerevents too

      var events = {
        'move': 'mousemove',
        'select': 'click',
        'selectstart': 'mousedown',
        'selectend': 'mouseup'
      };

      function onXRControllerEvent(event) {
        var controller = event.target;
        tempMatrix.identity().extractRotation(controller.matrixWorld);
        raycaster.ray.origin.setFromMatrixPosition(controller.matrixWorld);
        raycaster.ray.direction.set(0, 0, -1).applyMatrix4(tempMatrix);
        var intersections = raycaster.intersectObjects(scope.children, false);

        if (intersections.length > 0) {
          var intersection = intersections[0];
          var object = intersection.object;
          var uv = intersection.uv;
          _event.type = events[event.type];

          _event.data.set(uv.x, 1 - uv.y);

          object.dispatchEvent(_event);
        }
      }

      var controller1 = renderer.xr.getController(0);
      controller1.addEventListener('move', onXRControllerEvent);
      controller1.addEventListener('select', onXRControllerEvent);
      controller1.addEventListener('selectstart', onXRControllerEvent);
      controller1.addEventListener('selectend', onXRControllerEvent);
      var controller2 = renderer.xr.getController(1);
      controller2.addEventListener('move', onXRControllerEvent);
      controller2.addEventListener('select', onXRControllerEvent);
      controller2.addEventListener('selectstart', onXRControllerEvent);
      controller2.addEventListener('selectend', onXRControllerEvent);
      return _this;
    }

    return _createClass(InteractiveGroup);
  }(_three.Group);

  _exports.InteractiveGroup = InteractiveGroup;
});
