(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three);
    global.LUT3dlLoader = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.LUT3dlLoader = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var LUT3dlLoader = /*#__PURE__*/function (_Loader) {
    _inherits(LUT3dlLoader, _Loader);

    var _super = _createSuper(LUT3dlLoader);

    function LUT3dlLoader() {
      _classCallCheck(this, LUT3dlLoader);

      return _super.apply(this, arguments);
    }

    _createClass(LUT3dlLoader, [{
      key: "load",
      value: function load(url, onLoad, onProgress, onError) {
        var _this = this;

        var loader = new _three.FileLoader(this.manager);
        loader.setPath(this.path);
        loader.setResponseType('text');
        loader.load(url, function (text) {
          try {
            onLoad(_this.parse(text));
          } catch (e) {
            if (onError) {
              onError(e);
            } else {
              console.error(e);
            }

            _this.manager.itemError(url);
          }
        }, onProgress, onError);
      }
    }, {
      key: "parse",
      value: function parse(str) {
        // remove empty lines and comment lints
        str = str.replace(/^#.*?(\n|\r)/gm, '').replace(/^\s*?(\n|\r)/gm, '').trim();
        var lines = str.split(/[\n\r]+/g); // first line is the positions on the grid that are provided by the LUT

        var gridLines = lines[0].trim().split(/\s+/g).map(function (e) {
          return parseFloat(e);
        });
        var gridStep = gridLines[1] - gridLines[0];
        var size = gridLines.length;

        for (var i = 1, l = gridLines.length; i < l; i++) {
          if (gridStep !== gridLines[i] - gridLines[i - 1]) {
            throw new Error('LUT3dlLoader: Inconsistent grid size not supported.');
          }
        }

        var dataArray = new Array(size * size * size * 3);
        var index = 0;
        var maxOutputValue = 0.0;

        for (var _i = 1, _l = lines.length; _i < _l; _i++) {
          var line = lines[_i].trim();

          var split = line.split(/\s/g);
          var r = parseFloat(split[0]);
          var g = parseFloat(split[1]);
          var b = parseFloat(split[2]);
          maxOutputValue = Math.max(maxOutputValue, r, g, b);
          var bLayer = index % size;
          var gLayer = Math.floor(index / size) % size;
          var rLayer = Math.floor(index / (size * size)) % size; // b grows first, then g, then r

          var pixelIndex = bLayer * size * size + gLayer * size + rLayer;
          dataArray[3 * pixelIndex + 0] = r;
          dataArray[3 * pixelIndex + 1] = g;
          dataArray[3 * pixelIndex + 2] = b;
          index += 1;
        } // Find the apparent bit depth of the stored RGB values and scale the
        // values to [ 0, 255 ].


        var bits = Math.ceil(Math.log2(maxOutputValue));
        var maxBitValue = Math.pow(2.0, bits);

        for (var _i2 = 0, _l2 = dataArray.length; _i2 < _l2; _i2++) {
          var val = dataArray[_i2];
          dataArray[_i2] = 255 * val / maxBitValue;
        }

        var data = new Uint8Array(dataArray);
        var texture = new _three.DataTexture();
        texture.image.data = data;
        texture.image.width = size;
        texture.image.height = size * size;
        texture.format = _three.RGBFormat;
        texture.type = _three.UnsignedByteType;
        texture.magFilter = _three.LinearFilter;
        texture.minFilter = _three.LinearFilter;
        texture.wrapS = _three.ClampToEdgeWrapping;
        texture.wrapT = _three.ClampToEdgeWrapping;
        texture.generateMipmaps = false;
        var texture3D = new _three.DataTexture3D();
        texture3D.image.data = data;
        texture3D.image.width = size;
        texture3D.image.height = size;
        texture3D.image.depth = size;
        texture3D.format = _three.RGBFormat;
        texture3D.type = _three.UnsignedByteType;
        texture3D.magFilter = _three.LinearFilter;
        texture3D.minFilter = _three.LinearFilter;
        texture3D.wrapS = _three.ClampToEdgeWrapping;
        texture3D.wrapT = _three.ClampToEdgeWrapping;
        texture3D.wrapR = _three.ClampToEdgeWrapping;
        texture3D.generateMipmaps = false;
        return {
          size: size,
          texture: texture,
          texture3D: texture3D
        };
      }
    }]);

    return LUT3dlLoader;
  }(_three.Loader);

  _exports.LUT3dlLoader = LUT3dlLoader;
});
