(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "../core/TempNode.js", "../core/FunctionNode.js", "../inputs/FloatNode.js", "../inputs/Vector2Node.js", "../accessors/UVNode.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("../core/TempNode.js"), require("../core/FunctionNode.js"), require("../inputs/FloatNode.js"), require("../inputs/Vector2Node.js"), require("../accessors/UVNode.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.TempNode, global.FunctionNode, global.FloatNode, global.Vector2Node, global.UVNode);
    global.BlurNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, _TempNode2, _FunctionNode, _FloatNode, _Vector2Node, _UVNode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.BlurNode = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _get() { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(arguments.length < 3 ? target : receiver); } return desc.value; }; } return _get.apply(this, arguments); }

  function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var BlurNode = /*#__PURE__*/function (_TempNode) {
    _inherits(BlurNode, _TempNode);

    var _super = _createSuper(BlurNode);

    function BlurNode(value, uv, radius, size) {
      var _this;

      _classCallCheck(this, BlurNode);

      _this = _super.call(this, 'v4');
      _this.value = value;
      _this.uv = uv || new _UVNode.UVNode();
      _this.radius = radius || new _Vector2Node.Vector2Node(1, 1);
      _this.size = size;
      _this.blurX = true;
      _this.blurY = true;
      _this.horizontal = new _FloatNode.FloatNode(1 / 64);
      _this.vertical = new _FloatNode.FloatNode(1 / 64);
      return _this;
    }

    _createClass(BlurNode, [{
      key: "updateFrame",
      value: function
        /* frame */
      updateFrame() {
        if (this.size) {
          this.horizontal.value = this.radius.x / this.size.x;
          this.vertical.value = this.radius.y / this.size.y;
        } else if (this.value.value && this.value.value.image) {
          var image = this.value.value.image;
          this.horizontal.value = this.radius.x / image.width;
          this.vertical.value = this.radius.y / image.height;
        }
      }
    }, {
      key: "generate",
      value: function generate(builder, output) {
        if (builder.isShader('fragment')) {
          var blurCode = [];
          var code;
          var blurX = builder.include(BlurNode.Nodes.blurX),
              blurY = builder.include(BlurNode.Nodes.blurY);

          if (this.blurX) {
            blurCode.push(blurX + '( ' + this.value.build(builder, 'sampler2D') + ', ' + this.uv.build(builder, 'v2') + ', ' + this.horizontal.build(builder, 'f') + ' )');
          }

          if (this.blurY) {
            blurCode.push(blurY + '( ' + this.value.build(builder, 'sampler2D') + ', ' + this.uv.build(builder, 'v2') + ', ' + this.vertical.build(builder, 'f') + ' )');
          }

          if (blurCode.length == 2) code = '( ' + blurCode.join(' + ') + ' / 2.0 )';else if (blurCode.length) code = '( ' + blurCode[0] + ' )';else code = 'vec4( 0.0 )';
          return builder.format(code, this.getType(builder), output);
        } else {
          console.warn('THREE.BlurNode is not compatible with ' + builder.shader + ' shader.');
          return builder.format('vec4( 0.0 )', this.getType(builder), output);
        }
      }
    }, {
      key: "copy",
      value: function copy(source) {
        _get(_getPrototypeOf(BlurNode.prototype), "copy", this).call(this, source);

        this.value = source.value;
        this.uv = source.uv;
        this.radius = source.radius;
        if (source.size !== undefined) this.size = new _three.Vector2(source.size.x, source.size.y);
        this.blurX = source.blurX;
        this.blurY = source.blurY;
        return this;
      }
    }, {
      key: "toJSON",
      value: function toJSON(meta) {
        var data = this.getJSONNode(meta);

        if (!data) {
          data = this.createJSONNode(meta);
          data.value = this.value.toJSON(meta).uuid;
          data.uv = this.uv.toJSON(meta).uuid;
          data.radius = this.radius.toJSON(meta).uuid;
          if (this.size) data.size = {
            x: this.size.x,
            y: this.size.y
          };
          data.blurX = this.blurX;
          data.blurY = this.blurY;
        }

        return data;
      }
    }]);

    return BlurNode;
  }(_TempNode2.TempNode);

  _exports.BlurNode = BlurNode;

  BlurNode.Nodes = function () {
    var blurX = new _FunctionNode.FunctionNode(
    /* glsl */
    "\n\t\tvec4 blurX( sampler2D tex, vec2 uv, float s ) {\n\t\t\tvec4 sum = vec4( 0.0 );\n\t\t\tsum += texture2D( tex, vec2( uv.x - 4.0 * s, uv.y ) ) * 0.051;\n\t\t\tsum += texture2D( tex, vec2( uv.x - 3.0 * s, uv.y ) ) * 0.0918;\n\t\t\tsum += texture2D( tex, vec2( uv.x - 2.0 * s, uv.y ) ) * 0.12245;\n\t\t\tsum += texture2D( tex, vec2( uv.x - 1.0 * s, uv.y ) ) * 0.1531;\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y ) ) * 0.1633;\n\t\t\tsum += texture2D( tex, vec2( uv.x + 1.0 * s, uv.y ) ) * 0.1531;\n\t\t\tsum += texture2D( tex, vec2( uv.x + 2.0 * s, uv.y ) ) * 0.12245;\n\t\t\tsum += texture2D( tex, vec2( uv.x + 3.0 * s, uv.y ) ) * 0.0918;\n\t\t\tsum += texture2D( tex, vec2( uv.x + 4.0 * s, uv.y ) ) * 0.051;\n\t\t\treturn sum * .667;\n\t\t}");
    var blurY = new _FunctionNode.FunctionNode(
    /* glsl */
    "\n\t\tvec4 blurY( sampler2D tex, vec2 uv, float s ) {\n\t\t\tvec4 sum = vec4( 0.0 );\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y - 4.0 * s ) ) * 0.051;\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y - 3.0 * s ) ) * 0.0918;\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y - 2.0 * s ) ) * 0.12245;\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y - 1.0 * s ) ) * 0.1531;\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y ) ) * 0.1633;\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y + 1.0 * s ) ) * 0.1531;\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y + 2.0 * s ) ) * 0.12245;\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y + 3.0 * s ) ) * 0.0918;\n\t\t\tsum += texture2D( tex, vec2( uv.x, uv.y + 4.0 * s ) ) * 0.051;\n\t\t\treturn sum * .667;\n\t\t}");
    return {
      blurX: blurX,
      blurY: blurY
    };
  }();

  BlurNode.prototype.nodeType = 'Blur';
  BlurNode.prototype.hashProperties = ['blurX', 'blurY'];
});
