(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/TempNode.js", "../core/FunctionNode.js", "./LuminanceNode.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/TempNode.js"), require("../core/FunctionNode.js"), require("./LuminanceNode.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.TempNode, global.FunctionNode, global.LuminanceNode);
    global.ColorAdjustmentNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _TempNode2, _FunctionNode, _LuminanceNode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.ColorAdjustmentNode = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _get() { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(arguments.length < 3 ? target : receiver); } return desc.value; }; } return _get.apply(this, arguments); }

  function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var ColorAdjustmentNode = /*#__PURE__*/function (_TempNode) {
    _inherits(ColorAdjustmentNode, _TempNode);

    var _super = _createSuper(ColorAdjustmentNode);

    function ColorAdjustmentNode(rgb, adjustment, method) {
      var _this;

      _classCallCheck(this, ColorAdjustmentNode);

      _this = _super.call(this, 'v3');
      _this.rgb = rgb;
      _this.adjustment = adjustment;
      _this.method = method || ColorAdjustmentNode.SATURATION;
      return _this;
    }

    _createClass(ColorAdjustmentNode, [{
      key: "generate",
      value: function generate(builder, output) {
        var rgb = this.rgb.build(builder, 'v3'),
            adjustment = this.adjustment.build(builder, 'f');

        switch (this.method) {
          case ColorAdjustmentNode.BRIGHTNESS:
            return builder.format('( ' + rgb + ' + ' + adjustment + ' )', this.getType(builder), output);
            break;

          case ColorAdjustmentNode.CONTRAST:
            return builder.format('( ' + rgb + ' * ' + adjustment + ' )', this.getType(builder), output);
            break;
        }

        var method = builder.include(ColorAdjustmentNode.Nodes[this.method]);
        return builder.format(method + '( ' + rgb + ', ' + adjustment + ' )', this.getType(builder), output);
      }
    }, {
      key: "copy",
      value: function copy(source) {
        _get(_getPrototypeOf(ColorAdjustmentNode.prototype), "copy", this).call(this, source);

        this.rgb = source.rgb;
        this.adjustment = source.adjustment;
        this.method = source.method;
        return this;
      }
    }, {
      key: "toJSON",
      value: function toJSON(meta) {
        var data = this.getJSONNode(meta);

        if (!data) {
          data = this.createJSONNode(meta);
          data.rgb = this.rgb.toJSON(meta).uuid;
          data.adjustment = this.adjustment.toJSON(meta).uuid;
          data.method = this.method;
        }

        return data;
      }
    }]);

    return ColorAdjustmentNode;
  }(_TempNode2.TempNode);

  _exports.ColorAdjustmentNode = ColorAdjustmentNode;

  ColorAdjustmentNode.Nodes = function () {
    var hue = new _FunctionNode.FunctionNode(
    /* glsl */
    "\n\t\tvec3 hue(vec3 rgb, float adjustment) {\n\n\t\t\tconst mat3 RGBtoYIQ = mat3(0.299, 0.587, 0.114, 0.595716, -0.274453, -0.321263, 0.211456, -0.522591, 0.311135);\n\t\t\tconst mat3 YIQtoRGB = mat3(1.0, 0.9563, 0.6210, 1.0, -0.2721, -0.6474, 1.0, -1.107, 1.7046);\n\n\t\t\tvec3 yiq = RGBtoYIQ * rgb;\n\n\t\t\tfloat hue = atan(yiq.z, yiq.y) + adjustment;\n\t\t\tfloat chroma = sqrt(yiq.z * yiq.z + yiq.y * yiq.y);\n\n\t\t\treturn YIQtoRGB * vec3(yiq.x, chroma * cos(hue), chroma * sin(hue));\n\n\t\t}"); // Algorithm from Chapter 16 of OpenGL Shading Language

    var saturation = new _FunctionNode.FunctionNode(
    /* glsl */
    "\n\t\tvec3 saturation(vec3 rgb, float adjustment) {\n\n\t\t\tvec3 intensity = vec3( luminance( rgb ) );\n\n\t\t\treturn mix( intensity, rgb, adjustment );\n\n\t\t}", [_LuminanceNode.LuminanceNode.Nodes.luminance]); // include LuminanceNode function
    // Shader by Evan Wallace adapted by @lo-th

    var vibrance = new _FunctionNode.FunctionNode(
    /* glsl */
    "\n\n\t\tvec3 vibrance(vec3 rgb, float adjustment) {\n\n\t\t\tfloat average = (rgb.r + rgb.g + rgb.b) / 3.0;\n\n\t\t\tfloat mx = max(rgb.r, max(rgb.g, rgb.b));\n\t\t\tfloat amt = (mx - average) * (-3.0 * adjustment);\n\n\t\t\treturn mix(rgb.rgb, vec3(mx), amt);\n\n\t\t}");
    return {
      hue: hue,
      saturation: saturation,
      vibrance: vibrance
    };
  }();

  ColorAdjustmentNode.SATURATION = 'saturation';
  ColorAdjustmentNode.HUE = 'hue';
  ColorAdjustmentNode.VIBRANCE = 'vibrance';
  ColorAdjustmentNode.BRIGHTNESS = 'brightness';
  ColorAdjustmentNode.CONTRAST = 'contrast';
  ColorAdjustmentNode.prototype.nodeType = 'ColorAdjustment';
  ColorAdjustmentNode.prototype.hashProperties = ['method'];
});
