(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/TempNode.js", "../core/InputNode.js", "../accessors/PositionNode.js", "../math/OperatorNode.js", "./TextureNode.js", "./Matrix4Node.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/TempNode.js"), require("../core/InputNode.js"), require("../accessors/PositionNode.js"), require("../math/OperatorNode.js"), require("./TextureNode.js"), require("./Matrix4Node.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.TempNode, global.InputNode, global.PositionNode, global.OperatorNode, global.TextureNode, global.Matrix4Node);
    global.ReflectorNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _TempNode2, _InputNode, _PositionNode, _OperatorNode, _TextureNode, _Matrix4Node) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.ReflectorNode = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var ReflectorNode = /*#__PURE__*/function (_TempNode) {
    _inherits(ReflectorNode, _TempNode);

    var _super = _createSuper(ReflectorNode);

    function ReflectorNode(mirror) {
      var _this;

      _classCallCheck(this, ReflectorNode);

      _this = _super.call(this, 'v4');
      if (mirror) _this.setMirror(mirror);
      return _this;
    }

    _createClass(ReflectorNode, [{
      key: "setMirror",
      value: function setMirror(mirror) {
        this.mirror = mirror;
        this.textureMatrix = new _Matrix4Node.Matrix4Node(this.mirror.material.uniforms.textureMatrix.value);
        this.localPosition = new _PositionNode.PositionNode(_PositionNode.PositionNode.LOCAL);
        this.uv = new _OperatorNode.OperatorNode(this.textureMatrix, this.localPosition, _OperatorNode.OperatorNode.MUL);
        this.uvResult = new _OperatorNode.OperatorNode(null, this.uv, _OperatorNode.OperatorNode.ADD);
        this.texture = new _TextureNode.TextureNode(this.mirror.material.uniforms.tDiffuse.value, this.uv, null, true);
      }
    }, {
      key: "generate",
      value: function generate(builder, output) {
        if (builder.isShader('fragment')) {
          this.uvResult.a = this.offset;
          this.texture.uv = this.offset ? this.uvResult : this.uv;

          if (output === 'sampler2D') {
            return this.texture.build(builder, output);
          }

          return builder.format(this.texture.build(builder, this.type), this.type, output);
        } else {
          console.warn('THREE.ReflectorNode is not compatible with ' + builder.shader + ' shader.');
          return builder.format('vec4( 0.0 )', this.type, output);
        }
      }
    }, {
      key: "copy",
      value: function copy(source) {
        _InputNode.InputNode.prototype.copy.call(this, source);

        this.scope.mirror = source.mirror;
        return this;
      }
    }, {
      key: "toJSON",
      value: function toJSON(meta) {
        var data = this.getJSONNode(meta);

        if (!data) {
          data = this.createJSONNode(meta);
          data.mirror = this.mirror.uuid;
          if (this.offset) data.offset = this.offset.toJSON(meta).uuid;
        }

        return data;
      }
    }]);

    return ReflectorNode;
  }(_TempNode2.TempNode);

  _exports.ReflectorNode = ReflectorNode;
  ReflectorNode.prototype.nodeType = 'Reflector';
});
