(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/InputNode.js", "../accessors/UVNode.js", "../utils/ColorSpaceNode.js", "../core/ExpressionNode.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/InputNode.js"), require("../accessors/UVNode.js"), require("../utils/ColorSpaceNode.js"), require("../core/ExpressionNode.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.InputNode, global.UVNode, global.ColorSpaceNode, global.ExpressionNode);
    global.TextureNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _InputNode2, _UVNode, _ColorSpaceNode, _ExpressionNode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.TextureNode = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _get() { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(arguments.length < 3 ? target : receiver); } return desc.value; }; } return _get.apply(this, arguments); }

  function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var TextureNode = /*#__PURE__*/function (_InputNode) {
    _inherits(TextureNode, _InputNode);

    var _super = _createSuper(TextureNode);

    function TextureNode(value, uv, bias, project) {
      var _this;

      _classCallCheck(this, TextureNode);

      _this = _super.call(this, 'v4', {
        shared: true
      });
      _this.value = value;
      _this.uv = uv || new _UVNode.UVNode();
      _this.bias = bias;
      _this.project = project !== undefined ? project : false;
      return _this;
    }

    _createClass(TextureNode, [{
      key: "getTexture",
      value: function getTexture(builder, output) {
        return _get(_getPrototypeOf(TextureNode.prototype), "generate", this).call(this, builder, output, this.value.uuid, 't');
      }
    }, {
      key: "generate",
      value: function generate(builder, output) {
        if (output === 'sampler2D') {
          return this.getTexture(builder, output);
        }

        var tex = this.getTexture(builder, output),
            uv = this.uv.build(builder, this.project ? 'v4' : 'v2');
        var bias = this.bias ? this.bias.build(builder, 'f') : undefined;

        if (bias === undefined && builder.context.bias) {
          bias = builder.context.bias.setTexture(this).build(builder, 'f');
        }

        var method, code;
        if (this.project) method = 'texture2DProj';else method = bias ? 'tex2DBias' : 'tex2D';
        if (bias) code = method + '( ' + tex + ', ' + uv + ', ' + bias + ' )';else code = method + '( ' + tex + ', ' + uv + ' )'; // add a custom context for fix incompatibility with the core
        // include ColorSpace function only for vertex shader (in fragment shader color space functions is added automatically by core)
        // this should be removed in the future
        // context.include is used to include or not functions if used FunctionNode
        // context.ignoreCache =: not create temp variables nodeT0..9 to optimize the code

        var context = {
          include: builder.isShader('vertex'),
          ignoreCache: true
        };
        var outputType = this.getType(builder);
        builder.addContext(context);
        this.colorSpace = this.colorSpace || new _ColorSpaceNode.ColorSpaceNode(new _ExpressionNode.ExpressionNode('', outputType));
        this.colorSpace.fromDecoding(builder.getTextureEncodingFromMap(this.value));
        this.colorSpace.input.parse(code);
        code = this.colorSpace.build(builder, outputType); // end custom context

        builder.removeContext();
        return builder.format(code, outputType, output);
      }
    }, {
      key: "copy",
      value: function copy(source) {
        _get(_getPrototypeOf(TextureNode.prototype), "copy", this).call(this, source);

        if (source.value) this.value = source.value;
        this.uv = source.uv;
        if (source.bias) this.bias = source.bias;
        if (source.project !== undefined) this.project = source.project;
        return this;
      }
    }, {
      key: "toJSON",
      value: function toJSON(meta) {
        var data = this.getJSONNode(meta);

        if (!data) {
          data = this.createJSONNode(meta);
          if (this.value) data.value = this.value.uuid;
          data.uv = this.uv.toJSON(meta).uuid;
          data.project = this.project;
          if (this.bias) data.bias = this.bias.toJSON(meta).uuid;
        }

        return data;
      }
    }]);

    return TextureNode;
  }(_InputNode2.InputNode);

  _exports.TextureNode = TextureNode;
  TextureNode.prototype.nodeType = 'Texture';
});
