(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "../core/NodeBuilder.js", "../inputs/ColorNode.js", "../accessors/PositionNode.js", "./nodes/RawNode.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("../core/NodeBuilder.js"), require("../inputs/ColorNode.js"), require("../accessors/PositionNode.js"), require("./nodes/RawNode.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.NodeBuilder, global.ColorNode, global.PositionNode, global.RawNode);
    global.NodeMaterial = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, _NodeBuilder, _ColorNode, _PositionNode, _RawNode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.NodeMaterial = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var NodeMaterial = /*#__PURE__*/function (_ShaderMaterial) {
    _inherits(NodeMaterial, _ShaderMaterial);

    var _super = _createSuper(NodeMaterial);

    function NodeMaterial(vertex, fragment) {
      var _this;

      _classCallCheck(this, NodeMaterial);

      _this = _super.call(this);
      _this.vertex = vertex || new _RawNode.RawNode(new _PositionNode.PositionNode(_PositionNode.PositionNode.PROJECTION));
      _this.fragment = fragment || new _RawNode.RawNode(new _ColorNode.ColorNode(0xFF0000));
      _this.updaters = [];
      _this.type = 'NodeMaterial';
      return _this;
    }

    _createClass(NodeMaterial, [{
      key: "properties",
      get: function get() {
        return this.fragment.properties;
      }
    }, {
      key: "needsUpdate",
      get: function get() {
        return this.needsCompile;
      },
      set: function set(value) {
        if (value === true) this.version++;
        this.needsCompile = value;
      }
    }, {
      key: "onBeforeCompile",
      value: function onBeforeCompile(shader, renderer) {
        this.build({
          renderer: renderer
        });
        shader.defines = this.defines;
        shader.uniforms = this.uniforms;
        shader.vertexShader = this.vertexShader;
        shader.fragmentShader = this.fragmentShader;
        shader.extensionDerivatives = this.extensions.derivatives === true;
        shader.extensionFragDepth = this.extensions.fragDepth === true;
        shader.extensionDrawBuffers = this.extensions.drawBuffers === true;
        shader.extensionShaderTextureLOD = this.extensions.shaderTextureLOD === true;
      }
    }, {
      key: "customProgramCacheKey",
      value: function customProgramCacheKey() {
        var hash = this.getHash();
        return hash;
      }
    }, {
      key: "getHash",
      value: function getHash() {
        var hash = '{';
        hash += '"vertex":' + this.vertex.getHash() + ',';
        hash += '"fragment":' + this.fragment.getHash();
        hash += '}';
        return hash;
      }
    }, {
      key: "updateFrame",
      value: function updateFrame(frame) {
        for (var i = 0; i < this.updaters.length; ++i) {
          frame.updateNode(this.updaters[i]);
        }
      }
    }, {
      key: "build",
      value: function build() {
        var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var builder = params.builder || new _NodeBuilder.NodeBuilder();
        builder.setMaterial(this, params.renderer);
        builder.build(this.vertex, this.fragment);
        this.vertexShader = builder.getCode('vertex');
        this.fragmentShader = builder.getCode('fragment');
        this.defines = builder.defines;
        this.uniforms = builder.uniforms;
        this.extensions = builder.extensions;
        this.updaters = builder.updaters;
        this.fog = builder.requires.fog;
        this.lights = builder.requires.lights;
        this.transparent = builder.requires.transparent || this.blending > _three.NormalBlending;
        return this;
      }
    }, {
      key: "copy",
      value: function copy(source) {
        var uuid = this.uuid;

        for (var name in source) {
          this[name] = source[name];
        }

        this.uuid = uuid;

        if (source.userData !== undefined) {
          this.userData = JSON.parse(JSON.stringify(source.userData));
        }

        return this;
      }
    }, {
      key: "toJSON",
      value: function toJSON(meta) {
        var isRootObject = meta === undefined || typeof meta === 'string';

        if (isRootObject) {
          meta = {
            nodes: {}
          };
        }

        if (meta && !meta.materials) meta.materials = {};

        if (!meta.materials[this.uuid]) {
          var data = {};
          data.uuid = this.uuid;
          data.type = this.type;
          meta.materials[data.uuid] = data;
          if (this.name !== '') data.name = this.name;
          if (this.size !== undefined) data.size = this.size;
          if (this.sizeAttenuation !== undefined) data.sizeAttenuation = this.sizeAttenuation;
          if (this.blending !== _three.NormalBlending) data.blending = this.blending;
          if (this.flatShading === true) data.flatShading = this.flatShading;
          if (this.side !== _three.FrontSide) data.side = this.side;
          if (this.vertexColors !== _three.NoColors) data.vertexColors = this.vertexColors;
          if (this.depthFunc !== _three.LessEqualDepth) data.depthFunc = this.depthFunc;
          if (this.depthTest === false) data.depthTest = this.depthTest;
          if (this.depthWrite === false) data.depthWrite = this.depthWrite;
          if (this.linewidth !== 1) data.linewidth = this.linewidth;
          if (this.dashSize !== undefined) data.dashSize = this.dashSize;
          if (this.gapSize !== undefined) data.gapSize = this.gapSize;
          if (this.scale !== undefined) data.scale = this.scale;
          if (this.dithering === true) data.dithering = true;
          if (this.wireframe === true) data.wireframe = this.wireframe;
          if (this.wireframeLinewidth > 1) data.wireframeLinewidth = this.wireframeLinewidth;
          if (this.wireframeLinecap !== 'round') data.wireframeLinecap = this.wireframeLinecap;
          if (this.wireframeLinejoin !== 'round') data.wireframeLinejoin = this.wireframeLinejoin;
          if (this.alphaTest > 0) data.alphaTest = this.alphaTest;
          if (this.premultipliedAlpha === true) data.premultipliedAlpha = this.premultipliedAlpha;
          if (this.visible === false) data.visible = false;
          if (JSON.stringify(this.userData) !== '{}') data.userData = this.userData;
          data.fog = this.fog;
          data.lights = this.lights;
          data.vertex = this.vertex.toJSON(meta).uuid;
          data.fragment = this.fragment.toJSON(meta).uuid;
        }

        meta.material = this.uuid;
        return meta;
      }
    }]);

    return NodeMaterial;
  }(_three.ShaderMaterial);

  _exports.NodeMaterial = NodeMaterial;
  NodeMaterial.prototype.isNodeMaterial = true;
});
