(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/TempNode.js", "../inputs/FloatNode.js", "../core/FunctionNode.js", "../accessors/NormalNode.js", "../accessors/PositionNode.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/TempNode.js"), require("../inputs/FloatNode.js"), require("../core/FunctionNode.js"), require("../accessors/NormalNode.js"), require("../accessors/PositionNode.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.TempNode, global.FloatNode, global.FunctionNode, global.NormalNode, global.PositionNode);
    global.BumpMapNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _TempNode2, _FloatNode, _FunctionNode, _NormalNode, _PositionNode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.BumpMapNode = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _get() { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(arguments.length < 3 ? target : receiver); } return desc.value; }; } return _get.apply(this, arguments); }

  function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var BumpMapNode = /*#__PURE__*/function (_TempNode) {
    _inherits(BumpMapNode, _TempNode);

    var _super = _createSuper(BumpMapNode);

    function BumpMapNode(value, scale) {
      var _this;

      _classCallCheck(this, BumpMapNode);

      _this = _super.call(this, 'v3');
      _this.value = value;
      _this.scale = scale || new _FloatNode.FloatNode(1);
      _this.toNormalMap = false;
      return _this;
    }

    _createClass(BumpMapNode, [{
      key: "generate",
      value: function generate(builder, output) {
        if (builder.isShader('fragment')) {
          if (this.toNormalMap) {
            var bumpToNormal = builder.include(BumpMapNode.Nodes.bumpToNormal);
            return builder.format(bumpToNormal + '( ' + this.value.build(builder, 'sampler2D') + ', ' + this.value.uv.build(builder, 'v2') + ', ' + this.scale.build(builder, 'f') + ' )', this.getType(builder), output);
          } else {
            var derivativeHeight = builder.include(BumpMapNode.Nodes.dHdxy_fwd),
                perturbNormalArb = builder.include(BumpMapNode.Nodes.perturbNormalArb);
            this.normal = this.normal || new _NormalNode.NormalNode();
            this.position = this.position || new _PositionNode.PositionNode(_PositionNode.PositionNode.VIEW);
            var derivativeHeightCode = derivativeHeight + '( ' + this.value.build(builder, 'sampler2D') + ', ' + this.value.uv.build(builder, 'v2') + ', ' + this.scale.build(builder, 'f') + ' )';
            return builder.format(perturbNormalArb + '( -' + this.position.build(builder, 'v3') + ', ' + this.normal.build(builder, 'v3') + ', ' + derivativeHeightCode + ' )', this.getType(builder), output);
          }
        } else {
          console.warn('THREE.BumpMapNode is not compatible with ' + builder.shader + ' shader.');
          return builder.format('vec3( 0.0 )', this.getType(builder), output);
        }
      }
    }, {
      key: "copy",
      value: function copy(source) {
        _get(_getPrototypeOf(BumpMapNode.prototype), "copy", this).call(this, source);

        this.value = source.value;
        this.scale = source.scale;
        return this;
      }
    }, {
      key: "toJSON",
      value: function toJSON(meta) {
        var data = this.getJSONNode(meta);

        if (!data) {
          data = this.createJSONNode(meta);
          data.value = this.value.toJSON(meta).uuid;
          data.scale = this.scale.toJSON(meta).uuid;
        }

        return data;
      }
    }]);

    return BumpMapNode;
  }(_TempNode2.TempNode);

  _exports.BumpMapNode = BumpMapNode;

  BumpMapNode.Nodes = function () {
    // Bump Mapping Unparametrized Surfaces on the GPU by Morten S. Mikkelsen
    // http://api.unrealengine.com/attachments/Engine/Rendering/LightingAndShadows/BumpMappingWithoutTangentSpace/mm_sfgrad_bump.pdf
    // Evaluate the derivative of the height w.r.t. screen-space using forward differencing (listing 2)
    var dHdxy_fwd = new _FunctionNode.FunctionNode(
    /* glsl */
    "\n\n\t\tvec2 dHdxy_fwd( sampler2D bumpMap, vec2 vUv, float bumpScale ) {\n\n\t\t\tvec2 dSTdx = dFdx( vUv );\n\t\t\tvec2 dSTdy = dFdy( vUv );\n\n\t\t\tfloat Hll = bumpScale * texture2D( bumpMap, vUv ).x;\n\t\t\tfloat dBx = bumpScale * texture2D( bumpMap, vUv + dSTdx ).x - Hll;\n\t\t\tfloat dBy = bumpScale * texture2D( bumpMap, vUv + dSTdy ).x - Hll;\n\n\t\t\treturn vec2( dBx, dBy );\n\n\t\t}", null, {
      derivatives: true
    });
    var perturbNormalArb = new _FunctionNode.FunctionNode(
    /* glsl */
    "\n\n\t\tvec3 perturbNormalArb( vec3 surf_pos, vec3 surf_norm, vec2 dHdxy ) {\n\n\t\t\tvec3 vSigmaX = vec3( dFdx( surf_pos.x ), dFdx( surf_pos.y ), dFdx( surf_pos.z ) );\n\t\t\tvec3 vSigmaY = vec3( dFdy( surf_pos.x ), dFdy( surf_pos.y ), dFdy( surf_pos.z ) );\n\t\t\tvec3 vN = surf_norm; // normalized\n\n\t\t\tvec3 R1 = cross( vSigmaY, vN );\n\t\t\tvec3 R2 = cross( vN, vSigmaX );\n\n\t\t\tfloat fDet = dot( vSigmaX, R1 );\n\n\t\t\tfDet *= ( float( gl_FrontFacing ) * 2.0 - 1.0 );\n\n\t\t\tvec3 vGrad = sign( fDet ) * ( dHdxy.x * R1 + dHdxy.y * R2 );\n\n\t\t\treturn normalize( abs( fDet ) * surf_norm - vGrad );\n\n\t\t}", [dHdxy_fwd], {
      derivatives: true
    });
    var bumpToNormal = new _FunctionNode.FunctionNode(
    /* glsl */
    "\n\t\tvec3 bumpToNormal( sampler2D bumpMap, vec2 uv, float scale ) {\n\n\t\t\tvec2 dSTdx = dFdx( uv );\n\t\t\tvec2 dSTdy = dFdy( uv );\n\n\t\t\tfloat Hll = texture2D( bumpMap, uv ).x;\n\t\t\tfloat dBx = texture2D( bumpMap, uv + dSTdx ).x - Hll;\n\t\t\tfloat dBy = texture2D( bumpMap, uv + dSTdy ).x - Hll;\n\n\t\t\treturn vec3( .5 - ( dBx * scale ), .5 - ( dBy * scale ), 1.0 );\n\n\t\t}", null, {
      derivatives: true
    });
    return {
      dHdxy_fwd: dHdxy_fwd,
      perturbNormalArb: perturbNormalArb,
      bumpToNormal: bumpToNormal
    };
  }();

  BumpMapNode.prototype.nodeType = 'BumpMap';
  BumpMapNode.prototype.hashProperties = ['toNormalMap'];
});
