(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/TempNode.js", "../inputs/FloatNode.js", "./TextureCubeUVNode.js", "../accessors/ReflectNode.js", "../accessors/NormalNode.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/TempNode.js"), require("../inputs/FloatNode.js"), require("./TextureCubeUVNode.js"), require("../accessors/ReflectNode.js"), require("../accessors/NormalNode.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.TempNode, global.FloatNode, global.TextureCubeUVNode, global.ReflectNode, global.NormalNode);
    global.TextureCubeNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _TempNode2, _FloatNode, _TextureCubeUVNode, _ReflectNode, _NormalNode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.TextureCubeNode = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _get() { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(arguments.length < 3 ? target : receiver); } return desc.value; }; } return _get.apply(this, arguments); }

  function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var TextureCubeNode = /*#__PURE__*/function (_TempNode) {
    _inherits(TextureCubeNode, _TempNode);

    var _super = _createSuper(TextureCubeNode);

    function TextureCubeNode(value, uv, bias) {
      var _this;

      _classCallCheck(this, TextureCubeNode);

      _this = _super.call(this, 'v4');
      _this.value = value;
      _this.radianceNode = new _TextureCubeUVNode.TextureCubeUVNode(_this.value, uv || new _ReflectNode.ReflectNode(_ReflectNode.ReflectNode.VECTOR), // bias should be replaced in builder.context in build process
      bias);
      _this.irradianceNode = new _TextureCubeUVNode.TextureCubeUVNode(_this.value, new _NormalNode.NormalNode(_NormalNode.NormalNode.WORLD), new _FloatNode.FloatNode(1).setReadonly(true));
      return _this;
    }

    _createClass(TextureCubeNode, [{
      key: "generate",
      value: function generate(builder, output) {
        if (builder.isShader('fragment')) {
          builder.require('irradiance');

          if (builder.context.bias) {
            builder.context.bias.setTexture(this.value);
          }

          var scopeNode = builder.slot === 'irradiance' ? this.irradianceNode : this.radianceNode;
          return scopeNode.build(builder, output);
        } else {
          console.warn('THREE.TextureCubeNode is not compatible with ' + builder.shader + ' shader.');
          return builder.format('vec4( 0.0 )', this.getType(builder), output);
        }
      }
    }, {
      key: "copy",
      value: function copy(source) {
        _get(_getPrototypeOf(TextureCubeNode.prototype), "copy", this).call(this, source);

        this.value = source.value;
        return this;
      }
    }, {
      key: "toJSON",
      value: function toJSON(meta) {
        var data = this.getJSONNode(meta);

        if (!data) {
          data = this.createJSONNode(meta);
          data.value = this.value.toJSON(meta).uuid;
        }

        return data;
      }
    }]);

    return TextureCubeNode;
  }(_TempNode2.TempNode);

  _exports.TextureCubeNode = TextureCubeNode;
  TextureCubeNode.prototype.nodeType = 'TextureCube';
});
