(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/TempNode.js", "../core/FunctionNode.js", "../core/FunctionCallNode.js", "../inputs/IntNode.js", "../inputs/FloatNode.js", "../accessors/PositionNode.js", "./Noise3DNode.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/TempNode.js"), require("../core/FunctionNode.js"), require("../core/FunctionCallNode.js"), require("../inputs/IntNode.js"), require("../inputs/FloatNode.js"), require("../accessors/PositionNode.js"), require("./Noise3DNode.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.TempNode, global.FunctionNode, global.FunctionCallNode, global.IntNode, global.FloatNode, global.PositionNode, global.Noise3DNode);
    global.Fractal3DNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _TempNode2, _FunctionNode, _FunctionCallNode, _IntNode, _FloatNode, _PositionNode, _Noise3DNode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.Fractal3DNode = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _get() { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(arguments.length < 3 ? target : receiver); } return desc.value; }; } return _get.apply(this, arguments); }

  function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var FRACTAL3D_SRC = "\nfloat fractal3d( vec3 p, float amplitude, int octaves, float lacunarity, float diminish ) {\n\n\tfloat result = 0.0;\n\n\tfor (int i = 0;  i < octaves; ++i) {\n\n\t\tresult += noise3d(p, amplitude, 0.0);\n\t\tamplitude *= diminish;\n\t\tp *= lacunarity;\n\n\t}\n\n\treturn result;\n\n}\n".trim();
  /** Fractional Brownian motion. */

  var Fractal3DNode = /*#__PURE__*/function (_TempNode) {
    _inherits(Fractal3DNode, _TempNode);

    var _super = _createSuper(Fractal3DNode);

    function Fractal3DNode() {
      var _this;

      var position = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new _PositionNode.PositionNode();
      var amplitude = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _FloatNode.FloatNode(1.0);
      var octaves = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 3.0;
      var lacunarity = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 2.0;
      var diminish = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0.5;

      _classCallCheck(this, Fractal3DNode);

      _this = _super.call(this, 'f');
      _this.position = position;
      _this.amplitude = amplitude;
      _this.octaves = new _IntNode.IntNode(octaves).setReadonly(true);
      _this.lacunarity = new _FloatNode.FloatNode(lacunarity).setReadonly(true);
      _this.diminish = new _FloatNode.FloatNode(diminish).setReadonly(true);
      return _this;
    }

    _createClass(Fractal3DNode, [{
      key: "generate",
      value: function generate(builder, output) {
        var fractal3d = new _FunctionCallNode.FunctionCallNode(Fractal3DNode.Nodes.fractal3d, [this.position, this.amplitude, this.octaves, this.lacunarity, this.diminish]);
        return builder.format(fractal3d.generate(builder, output), this.getType(builder), output);
      }
    }, {
      key: "copy",
      value: function copy(source) {
        _get(_getPrototypeOf(Fractal3DNode.prototype), "copy", this).call(this, source);

        this.position = source.position;
        this.amplitude = source.amplitude;
        this.octaves = source.octaves;
        this.lacunarity = source.lacunarity;
        this.diminish = source.diminish;
      }
    }, {
      key: "toJSON",
      value: function toJSON(meta) {
        var data = this.getJSONNode(meta);

        if (!data) {
          data = this.createJSONNode(meta);
          data.position = this.position.toJSON(meta).uuid;
          data.amplitude = this.amplitude.toJSON(meta).uuid;
          data.octaves = this.octaves.toJSON(meta).uuid;
          data.lacunarity = this.lacunarity.toJSON(meta).uuid;
          data.diminish = this.diminish.toJSON(meta).uuid;
        }

        return data;
      }
    }]);

    return Fractal3DNode;
  }(_TempNode2.TempNode);

  _exports.Fractal3DNode = Fractal3DNode;
  Fractal3DNode.prototype.nodeType = 'Fractal3D';

  Fractal3DNode.Nodes = function () {
    var fractal3d = new _FunctionNode.FunctionNode(FRACTAL3D_SRC);
    fractal3d.includes = [_Noise3DNode.Noise3DNode.Nodes.noise3d];
    return {
      fractal3d: fractal3d
    };
  }();
});
