(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three);
    global.ShadowMesh = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.ShadowMesh = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  /**
   * A shadow Mesh that follows a shadow-casting Mesh in the scene, but is confined to a single plane.
   */
  var _shadowMatrix = new _three.Matrix4();

  var ShadowMesh = /*#__PURE__*/function (_Mesh) {
    _inherits(ShadowMesh, _Mesh);

    var _super = _createSuper(ShadowMesh);

    function ShadowMesh(mesh) {
      var _this;

      _classCallCheck(this, ShadowMesh);

      var shadowMaterial = new _three.MeshBasicMaterial({
        color: 0x000000,
        transparent: true,
        opacity: 0.6,
        depthWrite: false
      });
      _this = _super.call(this, mesh.geometry, shadowMaterial);
      _this.meshMatrix = mesh.matrixWorld;
      _this.frustumCulled = false;
      _this.matrixAutoUpdate = false;
      return _this;
    }

    _createClass(ShadowMesh, [{
      key: "update",
      value: function update(plane, lightPosition4D) {
        // based on https://www.opengl.org/archives/resources/features/StencilTalk/tsld021.htm
        var dot = plane.normal.x * lightPosition4D.x + plane.normal.y * lightPosition4D.y + plane.normal.z * lightPosition4D.z + -plane.constant * lightPosition4D.w;
        var sme = _shadowMatrix.elements;
        sme[0] = dot - lightPosition4D.x * plane.normal.x;
        sme[4] = -lightPosition4D.x * plane.normal.y;
        sme[8] = -lightPosition4D.x * plane.normal.z;
        sme[12] = -lightPosition4D.x * -plane.constant;
        sme[1] = -lightPosition4D.y * plane.normal.x;
        sme[5] = dot - lightPosition4D.y * plane.normal.y;
        sme[9] = -lightPosition4D.y * plane.normal.z;
        sme[13] = -lightPosition4D.y * -plane.constant;
        sme[2] = -lightPosition4D.z * plane.normal.x;
        sme[6] = -lightPosition4D.z * plane.normal.y;
        sme[10] = dot - lightPosition4D.z * plane.normal.z;
        sme[14] = -lightPosition4D.z * -plane.constant;
        sme[3] = -lightPosition4D.w * plane.normal.x;
        sme[7] = -lightPosition4D.w * plane.normal.y;
        sme[11] = -lightPosition4D.w * plane.normal.z;
        sme[15] = dot - lightPosition4D.w * -plane.constant;
        this.matrix.multiplyMatrices(_shadowMatrix, this.meshMatrix);
      }
    }]);

    return ShadowMesh;
  }(_three.Mesh);

  _exports.ShadowMesh = ShadowMesh;
  ShadowMesh.prototype.isShadowMesh = true;
});
