(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "../objects/Reflector.js", "../objects/Refractor.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("../objects/Reflector.js"), require("../objects/Refractor.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.Reflector, global.Refractor);
    global.Water2 = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, _Reflector, _Refractor) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.Water = void 0;

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  /**
   * References:
   *	http://www.valvesoftware.com/publications/2010/siggraph2010_vlachos_waterflow.pdf
   * 	http://graphicsrunner.blogspot.de/2010/08/water-using-flow-maps.html
   *
   */
  var Water = /*#__PURE__*/function (_Mesh) {
    _inherits(Water, _Mesh);

    var _super = _createSuper(Water);

    function Water(geometry) {
      var _this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      _classCallCheck(this, Water);

      _this = _super.call(this, geometry);
      _this.type = 'Water';

      var scope = _assertThisInitialized(_this);

      var color = options.color !== undefined ? new _three.Color(options.color) : new _three.Color(0xFFFFFF);
      var textureWidth = options.textureWidth || 512;
      var textureHeight = options.textureHeight || 512;
      var clipBias = options.clipBias || 0;
      var flowDirection = options.flowDirection || new _three.Vector2(1, 0);
      var flowSpeed = options.flowSpeed || 0.03;
      var reflectivity = options.reflectivity || 0.02;
      var scale = options.scale || 1;
      var shader = options.shader || Water.WaterShader;
      var encoding = options.encoding !== undefined ? options.encoding : _three.LinearEncoding;
      var textureLoader = new _three.TextureLoader();
      var flowMap = options.flowMap || undefined;
      var normalMap0 = options.normalMap0 || textureLoader.load('textures/water/Water_1_M_Normal.jpg');
      var normalMap1 = options.normalMap1 || textureLoader.load('textures/water/Water_2_M_Normal.jpg');
      var cycle = 0.15; // a cycle of a flow map phase

      var halfCycle = cycle * 0.5;
      var textureMatrix = new _three.Matrix4();
      var clock = new _three.Clock(); // internal components

      if (_Reflector.Reflector === undefined) {
        console.error('THREE.Water: Required component Reflector not found.');
        return _possibleConstructorReturn(_this);
      }

      if (_Refractor.Refractor === undefined) {
        console.error('THREE.Water: Required component Refractor not found.');
        return _possibleConstructorReturn(_this);
      }

      var reflector = new _Reflector.Reflector(geometry, {
        textureWidth: textureWidth,
        textureHeight: textureHeight,
        clipBias: clipBias,
        encoding: encoding
      });
      var refractor = new _Refractor.Refractor(geometry, {
        textureWidth: textureWidth,
        textureHeight: textureHeight,
        clipBias: clipBias,
        encoding: encoding
      });
      reflector.matrixAutoUpdate = false;
      refractor.matrixAutoUpdate = false; // material

      _this.material = new _three.ShaderMaterial({
        uniforms: _three.UniformsUtils.merge([_three.UniformsLib['fog'], shader.uniforms]),
        vertexShader: shader.vertexShader,
        fragmentShader: shader.fragmentShader,
        transparent: true,
        fog: true
      });

      if (flowMap !== undefined) {
        _this.material.defines.USE_FLOWMAP = '';
        _this.material.uniforms['tFlowMap'] = {
          type: 't',
          value: flowMap
        };
      } else {
        _this.material.uniforms['flowDirection'] = {
          type: 'v2',
          value: flowDirection
        };
      } // maps


      normalMap0.wrapS = normalMap0.wrapT = _three.RepeatWrapping;
      normalMap1.wrapS = normalMap1.wrapT = _three.RepeatWrapping;
      _this.material.uniforms['tReflectionMap'].value = reflector.getRenderTarget().texture;
      _this.material.uniforms['tRefractionMap'].value = refractor.getRenderTarget().texture;
      _this.material.uniforms['tNormalMap0'].value = normalMap0;
      _this.material.uniforms['tNormalMap1'].value = normalMap1; // water

      _this.material.uniforms['color'].value = color;
      _this.material.uniforms['reflectivity'].value = reflectivity;
      _this.material.uniforms['textureMatrix'].value = textureMatrix; // inital values

      _this.material.uniforms['config'].value.x = 0; // flowMapOffset0

      _this.material.uniforms['config'].value.y = halfCycle; // flowMapOffset1

      _this.material.uniforms['config'].value.z = halfCycle; // halfCycle

      _this.material.uniforms['config'].value.w = scale; // scale
      // functions

      function updateTextureMatrix(camera) {
        textureMatrix.set(0.5, 0.0, 0.0, 0.5, 0.0, 0.5, 0.0, 0.5, 0.0, 0.0, 0.5, 0.5, 0.0, 0.0, 0.0, 1.0);
        textureMatrix.multiply(camera.projectionMatrix);
        textureMatrix.multiply(camera.matrixWorldInverse);
        textureMatrix.multiply(scope.matrixWorld);
      }

      function updateFlow() {
        var delta = clock.getDelta();
        var config = scope.material.uniforms['config'];
        config.value.x += flowSpeed * delta; // flowMapOffset0

        config.value.y = config.value.x + halfCycle; // flowMapOffset1
        // Important: The distance between offsets should be always the value of "halfCycle".
        // Moreover, both offsets should be in the range of [ 0, cycle ].
        // This approach ensures a smooth water flow and avoids "reset" effects.

        if (config.value.x >= cycle) {
          config.value.x = 0;
          config.value.y = halfCycle;
        } else if (config.value.y >= cycle) {
          config.value.y = config.value.y - cycle;
        }
      } //


      _this.onBeforeRender = function (renderer, scene, camera) {
        updateTextureMatrix(camera);
        updateFlow();
        scope.visible = false;
        reflector.matrixWorld.copy(scope.matrixWorld);
        refractor.matrixWorld.copy(scope.matrixWorld);
        reflector.onBeforeRender(renderer, scene, camera);
        refractor.onBeforeRender(renderer, scene, camera);
        scope.visible = true;
      };

      return _this;
    }

    return _createClass(Water);
  }(_three.Mesh);

  _exports.Water = Water;
  Water.prototype.isWater = true;
  Water.WaterShader = {
    uniforms: {
      'color': {
        type: 'c',
        value: null
      },
      'reflectivity': {
        type: 'f',
        value: 0
      },
      'tReflectionMap': {
        type: 't',
        value: null
      },
      'tRefractionMap': {
        type: 't',
        value: null
      },
      'tNormalMap0': {
        type: 't',
        value: null
      },
      'tNormalMap1': {
        type: 't',
        value: null
      },
      'textureMatrix': {
        type: 'm4',
        value: null
      },
      'config': {
        type: 'v4',
        value: new _three.Vector4()
      }
    },
    vertexShader:
    /* glsl */
    "\n\n\t\t#include <common>\n\t\t#include <fog_pars_vertex>\n\t\t#include <logdepthbuf_pars_vertex>\n\n\t\tuniform mat4 textureMatrix;\n\n\t\tvarying vec4 vCoord;\n\t\tvarying vec2 vUv;\n\t\tvarying vec3 vToEye;\n\n\t\tvoid main() {\n\n\t\t\tvUv = uv;\n\t\t\tvCoord = textureMatrix * vec4( position, 1.0 );\n\n\t\t\tvec4 worldPosition = modelMatrix * vec4( position, 1.0 );\n\t\t\tvToEye = cameraPosition - worldPosition.xyz;\n\n\t\t\tvec4 mvPosition =  viewMatrix * worldPosition; // used in fog_vertex\n\t\t\tgl_Position = projectionMatrix * mvPosition;\n\n\t\t\t#include <logdepthbuf_vertex>\n\t\t\t#include <fog_vertex>\n\n\t\t}",
    fragmentShader:
    /* glsl */
    "\n\n\t\t#include <common>\n\t\t#include <fog_pars_fragment>\n\t\t#include <logdepthbuf_pars_fragment>\n\n\t\tuniform sampler2D tReflectionMap;\n\t\tuniform sampler2D tRefractionMap;\n\t\tuniform sampler2D tNormalMap0;\n\t\tuniform sampler2D tNormalMap1;\n\n\t\t#ifdef USE_FLOWMAP\n\t\t\tuniform sampler2D tFlowMap;\n\t\t#else\n\t\t\tuniform vec2 flowDirection;\n\t\t#endif\n\n\t\tuniform vec3 color;\n\t\tuniform float reflectivity;\n\t\tuniform vec4 config;\n\n\t\tvarying vec4 vCoord;\n\t\tvarying vec2 vUv;\n\t\tvarying vec3 vToEye;\n\n\t\tvoid main() {\n\n\t\t\t#include <logdepthbuf_fragment>\n\n\t\t\tfloat flowMapOffset0 = config.x;\n\t\t\tfloat flowMapOffset1 = config.y;\n\t\t\tfloat halfCycle = config.z;\n\t\t\tfloat scale = config.w;\n\n\t\t\tvec3 toEye = normalize( vToEye );\n\n\t\t\t// determine flow direction\n\t\t\tvec2 flow;\n\t\t\t#ifdef USE_FLOWMAP\n\t\t\t\tflow = texture2D( tFlowMap, vUv ).rg * 2.0 - 1.0;\n\t\t\t#else\n\t\t\t\tflow = flowDirection;\n\t\t\t#endif\n\t\t\tflow.x *= - 1.0;\n\n\t\t\t// sample normal maps (distort uvs with flowdata)\n\t\t\tvec4 normalColor0 = texture2D( tNormalMap0, ( vUv * scale ) + flow * flowMapOffset0 );\n\t\t\tvec4 normalColor1 = texture2D( tNormalMap1, ( vUv * scale ) + flow * flowMapOffset1 );\n\n\t\t\t// linear interpolate to get the final normal color\n\t\t\tfloat flowLerp = abs( halfCycle - flowMapOffset0 ) / halfCycle;\n\t\t\tvec4 normalColor = mix( normalColor0, normalColor1, flowLerp );\n\n\t\t\t// calculate normal vector\n\t\t\tvec3 normal = normalize( vec3( normalColor.r * 2.0 - 1.0, normalColor.b,  normalColor.g * 2.0 - 1.0 ) );\n\n\t\t\t// calculate the fresnel term to blend reflection and refraction maps\n\t\t\tfloat theta = max( dot( toEye, normal ), 0.0 );\n\t\t\tfloat reflectance = reflectivity + ( 1.0 - reflectivity ) * pow( ( 1.0 - theta ), 5.0 );\n\n\t\t\t// calculate final uv coords\n\t\t\tvec3 coord = vCoord.xyz / vCoord.w;\n\t\t\tvec2 uv = coord.xy + coord.z * normal.xz * 0.05;\n\n\t\t\tvec4 reflectColor = texture2D( tReflectionMap, vec2( 1.0 - uv.x, uv.y ) );\n\t\t\tvec4 refractColor = texture2D( tRefractionMap, uv );\n\n\t\t\t// multiply water color with the mix of both textures\n\t\t\tgl_FragColor = vec4( color, 1.0 ) * mix( refractColor, reflectColor, reflectance );\n\n\t\t\t#include <tonemapping_fragment>\n\t\t\t#include <encodings_fragment>\n\t\t\t#include <fog_fragment>\n\n\t\t}"
  };
});
