(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "./Pass.js", "../shaders/CopyShader.js", "../shaders/ConvolutionShader.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("./Pass.js"), require("../shaders/CopyShader.js"), require("../shaders/ConvolutionShader.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.Pass, global.CopyShader, global.ConvolutionShader);
    global.BloomPass = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, _Pass2, _CopyShader, _ConvolutionShader) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.BloomPass = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var BloomPass = /*#__PURE__*/function (_Pass) {
    _inherits(BloomPass, _Pass);

    var _super = _createSuper(BloomPass);

    function BloomPass() {
      var _this;

      var strength = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
      var kernelSize = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 25;
      var sigma = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 4;
      var resolution = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 256;

      _classCallCheck(this, BloomPass);

      _this = _super.call(this); // render targets

      var pars = {
        minFilter: _three.LinearFilter,
        magFilter: _three.LinearFilter,
        format: _three.RGBAFormat
      };
      _this.renderTargetX = new _three.WebGLRenderTarget(resolution, resolution, pars);
      _this.renderTargetX.texture.name = 'BloomPass.x';
      _this.renderTargetY = new _three.WebGLRenderTarget(resolution, resolution, pars);
      _this.renderTargetY.texture.name = 'BloomPass.y'; // copy material

      if (_CopyShader.CopyShader === undefined) console.error('THREE.BloomPass relies on CopyShader');
      var copyShader = _CopyShader.CopyShader;
      _this.copyUniforms = _three.UniformsUtils.clone(copyShader.uniforms);
      _this.copyUniforms['opacity'].value = strength;
      _this.materialCopy = new _three.ShaderMaterial({
        uniforms: _this.copyUniforms,
        vertexShader: copyShader.vertexShader,
        fragmentShader: copyShader.fragmentShader,
        blending: _three.AdditiveBlending,
        transparent: true
      }); // convolution material

      if (_ConvolutionShader.ConvolutionShader === undefined) console.error('THREE.BloomPass relies on ConvolutionShader');
      var convolutionShader = _ConvolutionShader.ConvolutionShader;
      _this.convolutionUniforms = _three.UniformsUtils.clone(convolutionShader.uniforms);
      _this.convolutionUniforms['uImageIncrement'].value = BloomPass.blurX;
      _this.convolutionUniforms['cKernel'].value = _ConvolutionShader.ConvolutionShader.buildKernel(sigma);
      _this.materialConvolution = new _three.ShaderMaterial({
        uniforms: _this.convolutionUniforms,
        vertexShader: convolutionShader.vertexShader,
        fragmentShader: convolutionShader.fragmentShader,
        defines: {
          'KERNEL_SIZE_FLOAT': kernelSize.toFixed(1),
          'KERNEL_SIZE_INT': kernelSize.toFixed(0)
        }
      });
      _this.needsSwap = false;
      _this.fsQuad = new _Pass2.FullScreenQuad(null);
      return _this;
    }

    _createClass(BloomPass, [{
      key: "render",
      value: function render(renderer, writeBuffer, readBuffer, deltaTime, maskActive) {
        if (maskActive) renderer.state.buffers.stencil.setTest(false); // Render quad with blured scene into texture (convolution pass 1)

        this.fsQuad.material = this.materialConvolution;
        this.convolutionUniforms['tDiffuse'].value = readBuffer.texture;
        this.convolutionUniforms['uImageIncrement'].value = BloomPass.blurX;
        renderer.setRenderTarget(this.renderTargetX);
        renderer.clear();
        this.fsQuad.render(renderer); // Render quad with blured scene into texture (convolution pass 2)

        this.convolutionUniforms['tDiffuse'].value = this.renderTargetX.texture;
        this.convolutionUniforms['uImageIncrement'].value = BloomPass.blurY;
        renderer.setRenderTarget(this.renderTargetY);
        renderer.clear();
        this.fsQuad.render(renderer); // Render original scene with superimposed blur to texture

        this.fsQuad.material = this.materialCopy;
        this.copyUniforms['tDiffuse'].value = this.renderTargetY.texture;
        if (maskActive) renderer.state.buffers.stencil.setTest(true);
        renderer.setRenderTarget(readBuffer);
        if (this.clear) renderer.clear();
        this.fsQuad.render(renderer);
      }
    }]);

    return BloomPass;
  }(_Pass2.Pass);

  _exports.BloomPass = BloomPass;
  BloomPass.blurX = new _three.Vector2(0.001953125, 0.0);
  BloomPass.blurY = new _three.Vector2(0.0, 0.001953125);
});
