(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "./Pass.js", "../shaders/HalftoneShader.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("./Pass.js"), require("../shaders/HalftoneShader.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.Pass, global.HalftoneShader);
    global.HalftonePass = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, _Pass2, _HalftoneShader) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.HalftonePass = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  /**
   * RGB Halftone pass for three.js effects composer. Requires HalftoneShader.
   */
  var HalftonePass = /*#__PURE__*/function (_Pass) {
    _inherits(HalftonePass, _Pass);

    var _super = _createSuper(HalftonePass);

    function HalftonePass(width, height, params) {
      var _this;

      _classCallCheck(this, HalftonePass);

      _this = _super.call(this);

      if (_HalftoneShader.HalftoneShader === undefined) {
        console.error('THREE.HalftonePass requires HalftoneShader');
      }

      _this.uniforms = _three.UniformsUtils.clone(_HalftoneShader.HalftoneShader.uniforms);
      _this.material = new _three.ShaderMaterial({
        uniforms: _this.uniforms,
        fragmentShader: _HalftoneShader.HalftoneShader.fragmentShader,
        vertexShader: _HalftoneShader.HalftoneShader.vertexShader
      }); // set params

      _this.uniforms.width.value = width;
      _this.uniforms.height.value = height;

      for (var key in params) {
        if (params.hasOwnProperty(key) && _this.uniforms.hasOwnProperty(key)) {
          _this.uniforms[key].value = params[key];
        }
      }

      _this.fsQuad = new _Pass2.FullScreenQuad(_this.material);
      return _this;
    }

    _createClass(HalftonePass, [{
      key: "render",
      value: function render(renderer, writeBuffer, readBuffer
      /*, deltaTime, maskActive*/
      ) {
        this.material.uniforms['tDiffuse'].value = readBuffer.texture;

        if (this.renderToScreen) {
          renderer.setRenderTarget(null);
          this.fsQuad.render(renderer);
        } else {
          renderer.setRenderTarget(writeBuffer);
          if (this.clear) renderer.clear();
          this.fsQuad.render(renderer);
        }
      }
    }, {
      key: "setSize",
      value: function setSize(width, height) {
        this.uniforms.width.value = width;
        this.uniforms.height.value = height;
      }
    }]);

    return HalftonePass;
  }(_Pass2.Pass);

  _exports.HalftonePass = HalftonePass;
});
