(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "./SSAARenderPass.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("./SSAARenderPass.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.SSAARenderPass);
    global.TAARenderPass = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, _SSAARenderPass2) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.TAARenderPass = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _get() { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(arguments.length < 3 ? target : receiver); } return desc.value; }; } return _get.apply(this, arguments); }

  function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  /**
   *
   * Temporal Anti-Aliasing Render Pass
   *
   * When there is no motion in the scene, the TAA render pass accumulates jittered camera samples across frames to create a high quality anti-aliased result.
   *
   * References:
   *
   * TODO: Add support for motion vector pas so that accumulation of samples across frames can occur on dynamics scenes.
   *
   */
  var TAARenderPass = /*#__PURE__*/function (_SSAARenderPass) {
    _inherits(TAARenderPass, _SSAARenderPass);

    var _super = _createSuper(TAARenderPass);

    function TAARenderPass(scene, camera, clearColor, clearAlpha) {
      var _this;

      _classCallCheck(this, TAARenderPass);

      _this = _super.call(this, scene, camera, clearColor, clearAlpha);
      _this.sampleLevel = 0;
      _this.accumulate = false;
      return _this;
    }

    _createClass(TAARenderPass, [{
      key: "render",
      value: function render(renderer, writeBuffer, readBuffer, deltaTime) {
        if (!this.accumulate) {
          _get(_getPrototypeOf(TAARenderPass.prototype), "render", this).call(this, renderer, writeBuffer, readBuffer, deltaTime);

          this.accumulateIndex = -1;
          return;
        }

        var jitterOffsets = _JitterVectors[5];

        if (!this.sampleRenderTarget) {
          this.sampleRenderTarget = new _three.WebGLRenderTarget(readBuffer.width, readBuffer.height, this.params);
          this.sampleRenderTarget.texture.name = 'TAARenderPass.sample';
        }

        if (!this.holdRenderTarget) {
          this.holdRenderTarget = new _three.WebGLRenderTarget(readBuffer.width, readBuffer.height, this.params);
          this.holdRenderTarget.texture.name = 'TAARenderPass.hold';
        }

        if (this.accumulate && this.accumulateIndex === -1) {
          _get(_getPrototypeOf(TAARenderPass.prototype), "render", this).call(this, renderer, this.holdRenderTarget, readBuffer, deltaTime);

          this.accumulateIndex = 0;
        }

        var autoClear = renderer.autoClear;
        renderer.autoClear = false;
        var sampleWeight = 1.0 / jitterOffsets.length;

        if (this.accumulateIndex >= 0 && this.accumulateIndex < jitterOffsets.length) {
          this.copyUniforms['opacity'].value = sampleWeight;
          this.copyUniforms['tDiffuse'].value = writeBuffer.texture; // render the scene multiple times, each slightly jitter offset from the last and accumulate the results.

          var numSamplesPerFrame = Math.pow(2, this.sampleLevel);

          for (var i = 0; i < numSamplesPerFrame; i++) {
            var j = this.accumulateIndex;
            var jitterOffset = jitterOffsets[j];

            if (this.camera.setViewOffset) {
              this.camera.setViewOffset(readBuffer.width, readBuffer.height, jitterOffset[0] * 0.0625, jitterOffset[1] * 0.0625, // 0.0625 = 1 / 16
              readBuffer.width, readBuffer.height);
            }

            renderer.setRenderTarget(writeBuffer);
            renderer.clear();
            renderer.render(this.scene, this.camera);
            renderer.setRenderTarget(this.sampleRenderTarget);
            if (this.accumulateIndex === 0) renderer.clear();
            this.fsQuad.render(renderer);
            this.accumulateIndex++;
            if (this.accumulateIndex >= jitterOffsets.length) break;
          }

          if (this.camera.clearViewOffset) this.camera.clearViewOffset();
        }

        var accumulationWeight = this.accumulateIndex * sampleWeight;

        if (accumulationWeight > 0) {
          this.copyUniforms['opacity'].value = 1.0;
          this.copyUniforms['tDiffuse'].value = this.sampleRenderTarget.texture;
          renderer.setRenderTarget(writeBuffer);
          renderer.clear();
          this.fsQuad.render(renderer);
        }

        if (accumulationWeight < 1.0) {
          this.copyUniforms['opacity'].value = 1.0 - accumulationWeight;
          this.copyUniforms['tDiffuse'].value = this.holdRenderTarget.texture;
          renderer.setRenderTarget(writeBuffer);
          if (accumulationWeight === 0) renderer.clear();
          this.fsQuad.render(renderer);
        }

        renderer.autoClear = autoClear;
      }
    }]);

    return TAARenderPass;
  }(_SSAARenderPass2.SSAARenderPass);

  _exports.TAARenderPass = TAARenderPass;
  var _JitterVectors = [[[0, 0]], [[4, 4], [-4, -4]], [[-2, -6], [6, -2], [-6, 2], [2, 6]], [[1, -3], [-1, 3], [5, 1], [-3, -5], [-5, 5], [-7, -1], [3, 7], [7, -7]], [[1, 1], [-1, -3], [-3, 2], [4, -1], [-5, -2], [2, 5], [5, 3], [3, -5], [-2, 6], [0, -7], [-4, -6], [-6, 4], [-8, 0], [7, -4], [6, 7], [-7, -8]], [[-4, -7], [-7, -5], [-3, -5], [-5, -4], [-1, -4], [-2, -2], [-6, -1], [-4, 0], [-7, 1], [-1, 2], [-6, 3], [-3, 3], [-7, 6], [-3, 6], [-5, 7], [-1, 7], [5, -7], [1, -6], [6, -5], [4, -4], [2, -3], [7, -2], [1, -1], [4, -1], [2, 1], [6, 2], [0, 4], [4, 4], [2, 5], [7, 5], [5, 6], [3, 7]]];
});
