(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/Node.js", "../math/OperatorNode.js", "./MaterialReferenceNode.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/Node.js"), require("../math/OperatorNode.js"), require("./MaterialReferenceNode.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.Node, global.OperatorNode, global.MaterialReferenceNode);
    global.MaterialNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _Node2, _OperatorNode, _MaterialReferenceNode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  _Node2 = _interopRequireDefault(_Node2);
  _OperatorNode = _interopRequireDefault(_OperatorNode);
  _MaterialReferenceNode = _interopRequireDefault(_MaterialReferenceNode);

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

  var MaterialNode = /*#__PURE__*/function (_Node) {
    _inherits(MaterialNode, _Node);

    var _super = _createSuper(MaterialNode);

    function MaterialNode() {
      var _this;

      var scope = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : MaterialNode.COLOR;

      _classCallCheck(this, MaterialNode);

      _this = _super.call(this);
      _this.scope = scope;
      return _this;
    }

    _createClass(MaterialNode, [{
      key: "getNodeType",
      value: function getNodeType(builder) {
        var scope = this.scope;
        var material = builder.context.material;

        if (scope === MaterialNode.COLOR) {
          return material.map !== null ? 'vec4' : 'vec3';
        } else if (scope === MaterialNode.OPACITY) {
          return 'float';
        } else if (scope === MaterialNode.SPECULAR) {
          return 'vec3';
        } else if (scope === MaterialNode.ROUGHNESS || scope === MaterialNode.METALNESS) {
          return 'float';
        }
      }
    }, {
      key: "generate",
      value: function generate(builder, output) {
        var material = builder.context.material;
        var scope = this.scope;
        var node = null;

        if (scope === MaterialNode.ALPHA_TEST) {
          node = new _MaterialReferenceNode.default('alphaTest', 'float');
        } else if (scope === MaterialNode.COLOR) {
          var colorNode = new _MaterialReferenceNode.default('color', 'color');

          if (material.map !== null && material.map !== undefined && material.map.isTexture === true) {
            node = new _OperatorNode.default('*', colorNode, new _MaterialReferenceNode.default('map', 'texture'));
          } else {
            node = colorNode;
          }
        } else if (scope === MaterialNode.OPACITY) {
          var opacityNode = new _MaterialReferenceNode.default('opacity', 'float');

          if (material.alphaMap !== null && material.alphaMap !== undefined && material.alphaMap.isTexture === true) {
            node = new _OperatorNode.default('*', opacityNode, new _MaterialReferenceNode.default('alphaMap', 'texture'));
          } else {
            node = opacityNode;
          }
        } else if (scope === MaterialNode.SPECULAR) {
          var specularColorNode = new _MaterialReferenceNode.default('specularColor', 'color');

          if (material.specularColorMap !== null && material.specularColorMap !== undefined && material.specularColorMap.isTexture === true) {
            node = new _OperatorNode.default('*', specularColorNode, new _MaterialReferenceNode.default('specularColorMap', 'texture'));
          } else {
            node = specularColorNode;
          }
        } else {
          var outputType = this.getNodeType(builder);
          node = new _MaterialReferenceNode.default(scope, outputType);
        }

        return node.build(builder, output);
      }
    }]);

    return MaterialNode;
  }(_Node2.default);

  _defineProperty(MaterialNode, "ALPHA_TEST", 'alphaTest');

  _defineProperty(MaterialNode, "COLOR", 'color');

  _defineProperty(MaterialNode, "OPACITY", 'opacity');

  _defineProperty(MaterialNode, "SPECULAR", 'specular');

  _defineProperty(MaterialNode, "ROUGHNESS", 'roughness');

  _defineProperty(MaterialNode, "METALNESS", 'metalness');

  var _default = MaterialNode;
  _exports.default = _default;
});
