(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/Node.js", "../core/AttributeNode.js", "../core/VaryNode.js", "../accessors/ModelNode.js", "../accessors/CameraNode.js", "../math/OperatorNode.js", "../math/MathNode.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/Node.js"), require("../core/AttributeNode.js"), require("../core/VaryNode.js"), require("../accessors/ModelNode.js"), require("../accessors/CameraNode.js"), require("../math/OperatorNode.js"), require("../math/MathNode.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.Node, global.AttributeNode, global.VaryNode, global.ModelNode, global.CameraNode, global.OperatorNode, global.MathNode);
    global.NormalNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _Node2, _AttributeNode, _VaryNode, _ModelNode, _CameraNode, _OperatorNode, _MathNode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  _Node2 = _interopRequireDefault(_Node2);
  _AttributeNode = _interopRequireDefault(_AttributeNode);
  _VaryNode = _interopRequireDefault(_VaryNode);
  _ModelNode = _interopRequireDefault(_ModelNode);
  _CameraNode = _interopRequireDefault(_CameraNode);
  _OperatorNode = _interopRequireDefault(_OperatorNode);
  _MathNode = _interopRequireDefault(_MathNode);

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

  var NormalNode = /*#__PURE__*/function (_Node) {
    _inherits(NormalNode, _Node);

    var _super = _createSuper(NormalNode);

    function NormalNode() {
      var _this;

      var scope = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : NormalNode.LOCAL;

      _classCallCheck(this, NormalNode);

      _this = _super.call(this, 'vec3');
      _this.scope = scope;
      return _this;
    }

    _createClass(NormalNode, [{
      key: "getHash",
      value: function
        /*builder*/
      getHash() {
        return "normal-".concat(this.scope);
      }
    }, {
      key: "generate",
      value: function generate(builder) {
        var scope = this.scope;
        var outputNode = null;

        if (scope === NormalNode.GEOMETRY) {
          outputNode = new _AttributeNode.default('normal', 'vec3');
        } else if (scope === NormalNode.LOCAL) {
          outputNode = new _VaryNode.default(new NormalNode(NormalNode.GEOMETRY));
        } else if (scope === NormalNode.VIEW) {
          var vertexNormalNode = new _OperatorNode.default('*', new _ModelNode.default(_ModelNode.default.NORMAL_MATRIX), new NormalNode(NormalNode.LOCAL));
          outputNode = new _MathNode.default(_MathNode.default.NORMALIZE, new _VaryNode.default(vertexNormalNode));
        } else if (scope === NormalNode.WORLD) {
          // To use INVERSE_TRANSFORM_DIRECTION only inverse the param order like this: MathNode( ..., Vector, Matrix );
          var _vertexNormalNode = new _MathNode.default(_MathNode.default.TRANSFORM_DIRECTION, new NormalNode(NormalNode.VIEW), new _CameraNode.default(_CameraNode.default.VIEW_MATRIX));

          outputNode = new _MathNode.default(_MathNode.default.NORMALIZE, new _VaryNode.default(_vertexNormalNode));
        }

        return outputNode.build(builder);
      }
    }]);

    return NormalNode;
  }(_Node2.default);

  _defineProperty(NormalNode, "GEOMETRY", 'geometry');

  _defineProperty(NormalNode, "LOCAL", 'local');

  _defineProperty(NormalNode, "WORLD", 'world');

  _defineProperty(NormalNode, "VIEW", 'view');

  var _default = NormalNode;
  _exports.default = _default;
});
