(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/Node.js", "../inputs/Matrix4Node.js", "../inputs/Matrix3Node.js", "../inputs/Vector3Node.js", "../core/constants.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/Node.js"), require("../inputs/Matrix4Node.js"), require("../inputs/Matrix3Node.js"), require("../inputs/Vector3Node.js"), require("../core/constants.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.Node, global.Matrix4Node, global.Matrix3Node, global.Vector3Node, global.constants);
    global.Object3DNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _Node2, _Matrix4Node, _Matrix3Node, _Vector3Node, _constants) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  _Node2 = _interopRequireDefault(_Node2);
  _Matrix4Node = _interopRequireDefault(_Matrix4Node);
  _Matrix3Node = _interopRequireDefault(_Matrix3Node);
  _Vector3Node = _interopRequireDefault(_Vector3Node);

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

  var Object3DNode = /*#__PURE__*/function (_Node) {
    _inherits(Object3DNode, _Node);

    var _super = _createSuper(Object3DNode);

    function Object3DNode() {
      var _this;

      var scope = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Object3DNode.VIEW_MATRIX;
      var object3d = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      _classCallCheck(this, Object3DNode);

      _this = _super.call(this);
      _this.scope = scope;
      _this.object3d = object3d;
      _this.updateType = _constants.NodeUpdateType.Object;
      _this._inputNode = null;
      return _this;
    }

    _createClass(Object3DNode, [{
      key: "getNodeType",
      value: function getNodeType() {
        var scope = this.scope;

        if (scope === Object3DNode.WORLD_MATRIX || scope === Object3DNode.VIEW_MATRIX) {
          return 'mat4';
        } else if (scope === Object3DNode.NORMAL_MATRIX) {
          return 'mat3';
        } else if (scope === Object3DNode.POSITION || scope === Object3DNode.VIEW_POSITION) {
          return 'vec3';
        }
      }
    }, {
      key: "update",
      value: function update(frame) {
        var object = this.object3d !== null ? this.object3d : frame.object;
        var inputNode = this._inputNode;
        var camera = frame.camera;
        var scope = this.scope;

        if (scope === Object3DNode.VIEW_MATRIX) {
          inputNode.value = object.modelViewMatrix;
        } else if (scope === Object3DNode.NORMAL_MATRIX) {
          inputNode.value = object.normalMatrix;
        } else if (scope === Object3DNode.WORLD_MATRIX) {
          inputNode.value = object.matrixWorld;
        } else if (scope === Object3DNode.POSITION) {
          inputNode.value.setFromMatrixPosition(object.matrixWorld);
        } else if (scope === Object3DNode.VIEW_POSITION) {
          inputNode.value.setFromMatrixPosition(object.matrixWorld);
          inputNode.value.applyMatrix4(camera.matrixWorldInverse);
        }
      }
    }, {
      key: "generate",
      value: function generate(builder) {
        var scope = this.scope;

        if (scope === Object3DNode.WORLD_MATRIX || scope === Object3DNode.VIEW_MATRIX) {
          this._inputNode = new _Matrix4Node.default();
        } else if (scope === Object3DNode.NORMAL_MATRIX) {
          this._inputNode = new _Matrix3Node.default();
        } else if (scope === Object3DNode.POSITION || scope === Object3DNode.VIEW_POSITION) {
          this._inputNode = new _Vector3Node.default();
        }

        return this._inputNode.build(builder);
      }
    }]);

    return Object3DNode;
  }(_Node2.default);

  _defineProperty(Object3DNode, "VIEW_MATRIX", 'viewMatrix');

  _defineProperty(Object3DNode, "NORMAL_MATRIX", 'normalMatrix');

  _defineProperty(Object3DNode, "WORLD_MATRIX", 'worldMatrix');

  _defineProperty(Object3DNode, "POSITION", 'position');

  _defineProperty(Object3DNode, "VIEW_POSITION", 'viewPosition');

  var _default = Object3DNode;
  _exports.default = _default;
});
