(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/Node.js", "../inputs/FloatNode.js", "../inputs/Vector2Node.js", "../inputs/Vector3Node.js", "../inputs/Vector4Node.js", "../inputs/ColorNode.js", "../inputs/TextureNode.js", "../core/constants.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/Node.js"), require("../inputs/FloatNode.js"), require("../inputs/Vector2Node.js"), require("../inputs/Vector3Node.js"), require("../inputs/Vector4Node.js"), require("../inputs/ColorNode.js"), require("../inputs/TextureNode.js"), require("../core/constants.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.Node, global.FloatNode, global.Vector2Node, global.Vector3Node, global.Vector4Node, global.ColorNode, global.TextureNode, global.constants);
    global.ReferenceNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _Node2, _FloatNode, _Vector2Node, _Vector3Node, _Vector4Node, _ColorNode, _TextureNode, _constants) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  _Node2 = _interopRequireDefault(_Node2);
  _FloatNode = _interopRequireDefault(_FloatNode);
  _Vector2Node = _interopRequireDefault(_Vector2Node);
  _Vector3Node = _interopRequireDefault(_Vector3Node);
  _Vector4Node = _interopRequireDefault(_Vector4Node);
  _ColorNode = _interopRequireDefault(_ColorNode);
  _TextureNode = _interopRequireDefault(_TextureNode);

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var ReferenceNode = /*#__PURE__*/function (_Node) {
    _inherits(ReferenceNode, _Node);

    var _super = _createSuper(ReferenceNode);

    function ReferenceNode(property, inputType) {
      var _this;

      var object = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

      _classCallCheck(this, ReferenceNode);

      _this = _super.call(this);
      _this.property = property;
      _this.inputType = inputType;
      _this.object = object;
      _this.node = null;
      _this.updateType = _constants.NodeUpdateType.Object;

      _this.setNodeType(inputType);

      return _this;
    }

    _createClass(ReferenceNode, [{
      key: "setNodeType",
      value: function setNodeType(inputType) {
        var node = null;
        var nodeType = inputType;

        if (nodeType === 'float') {
          node = new _FloatNode.default();
        } else if (nodeType === 'vec2') {
          node = new _Vector2Node.default(null);
        } else if (nodeType === 'vec3') {
          node = new _Vector3Node.default(null);
        } else if (nodeType === 'vec4') {
          node = new _Vector4Node.default(null);
        } else if (nodeType === 'color') {
          node = new _ColorNode.default(null);
          nodeType = 'vec3';
        } else if (nodeType === 'texture') {
          node = new _TextureNode.default();
          nodeType = 'vec4';
        }

        this.node = node;
        this.nodeType = nodeType;
        this.inputType = inputType;
      }
    }, {
      key: "getNodeType",
      value: function getNodeType() {
        return this.inputType;
      }
    }, {
      key: "update",
      value: function update(frame) {
        var object = this.object !== null ? this.object : frame.object;
        var value = object[this.property];
        this.node.value = value;
      }
    }, {
      key: "generate",
      value: function generate(builder) {
        return this.node.build(builder, this.getNodeType(builder));
      }
    }]);

    return ReferenceNode;
  }(_Node2.default);

  var _default = ReferenceNode;
  _exports.default = _default;
});
