(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../accessors/PositionNode.js", "../accessors/NormalNode.js", "../accessors/UVNode.js", "../math/MathNode.js", "../math/OperatorNode.js", "../inputs/FloatNode.js", "../core/TempNode.js", "../accessors/ModelNode.js", "../ShaderNode.js", "three"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../accessors/PositionNode.js"), require("../accessors/NormalNode.js"), require("../accessors/UVNode.js"), require("../math/MathNode.js"), require("../math/OperatorNode.js"), require("../inputs/FloatNode.js"), require("../core/TempNode.js"), require("../accessors/ModelNode.js"), require("../ShaderNode.js"), require("three"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.PositionNode, global.NormalNode, global.UVNode, global.MathNode, global.OperatorNode, global.FloatNode, global.TempNode, global.ModelNode, global.ShaderNode, global.three);
    global.NormalMapNode = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _PositionNode, _NormalNode, _UVNode, _MathNode, _OperatorNode, _FloatNode, _TempNode2, _ModelNode, _ShaderNode, _three) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  _PositionNode = _interopRequireDefault(_PositionNode);
  _NormalNode = _interopRequireDefault(_NormalNode);
  _UVNode = _interopRequireDefault(_UVNode);
  _MathNode = _interopRequireDefault(_MathNode);
  _OperatorNode = _interopRequireDefault(_OperatorNode);
  _FloatNode = _interopRequireDefault(_FloatNode);
  _TempNode2 = _interopRequireDefault(_TempNode2);
  _ModelNode = _interopRequireDefault(_ModelNode);

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  // Normal Mapping Without Precomputed Tangents
  // http://www.thetenthplanet.de/archives/1180
  var perturbNormal2ArbNode = new _ShaderNode.ShaderNode(function (inputs) {
    var eye_pos = inputs.eye_pos,
        surf_norm = inputs.surf_norm,
        mapN = inputs.mapN,
        faceDirection = inputs.faceDirection,
        uv = inputs.uv;
    var q0 = (0, _ShaderNode.dFdx)(eye_pos.xyz);
    var q1 = (0, _ShaderNode.dFdy)(eye_pos.xyz);
    var st0 = (0, _ShaderNode.dFdx)(uv.st);
    var st1 = (0, _ShaderNode.dFdy)(uv.st);
    var N = surf_norm; // normalized

    var q1perp = (0, _ShaderNode.cross)(q1, N);
    var q0perp = (0, _ShaderNode.cross)(N, q0);
    var T = (0, _ShaderNode.add)((0, _ShaderNode.mul)(q1perp, st0.x), (0, _ShaderNode.mul)(q0perp, st1.x));
    var B = (0, _ShaderNode.add)((0, _ShaderNode.mul)(q1perp, st0.y), (0, _ShaderNode.mul)(q0perp, st1.y));
    var det = (0, _ShaderNode.max)((0, _ShaderNode.dot)(T, T), (0, _ShaderNode.dot)(B, B));
    var scale = (0, _ShaderNode.cond)((0, _ShaderNode.equal)(det, 0), 0, (0, _ShaderNode.mul)(faceDirection, (0, _ShaderNode.inversesqrt)(det)));
    return (0, _ShaderNode.normalize)((0, _ShaderNode.add)((0, _ShaderNode.mul)(T, (0, _ShaderNode.mul)(mapN.x, scale)), (0, _ShaderNode.mul)(B, (0, _ShaderNode.mul)(mapN.y, scale)), (0, _ShaderNode.mul)(N, mapN.z)));
  });

  var NormalMapNode = /*#__PURE__*/function (_TempNode) {
    _inherits(NormalMapNode, _TempNode);

    var _super = _createSuper(NormalMapNode);

    function NormalMapNode(node) {
      var _this;

      _classCallCheck(this, NormalMapNode);

      _this = _super.call(this, 'vec3');
      _this.node = node;
      _this.normalMapType = _three.TangentSpaceNormalMap;
      return _this;
    }

    _createClass(NormalMapNode, [{
      key: "generate",
      value: function generate(builder) {
        var type = this.getNodeType(builder);
        var normalMapType = this.normalMapType;
        var normalOP = new _OperatorNode.default('*', this.node, new _FloatNode.default(2.0).setConst(true));
        var normalMap = new _OperatorNode.default('-', normalOP, new _FloatNode.default(1.0).setConst(true));

        if (normalMapType === _three.ObjectSpaceNormalMap) {
          var vertexNormalNode = new _OperatorNode.default('*', new _ModelNode.default(_ModelNode.default.NORMAL_MATRIX), normalMap);
          var normal = new _MathNode.default(_MathNode.default.NORMALIZE, vertexNormalNode);
          return normal.build(builder, type);
        } else if (normalMapType === _three.TangentSpaceNormalMap) {
          var perturbNormal2ArbCall = perturbNormal2ArbNode({
            eye_pos: new _PositionNode.default(_PositionNode.default.VIEW),
            surf_norm: new _NormalNode.default(_NormalNode.default.VIEW),
            mapN: normalMap,
            faceDirection: new _FloatNode.default(1.0).setConst(true),
            uv: new _UVNode.default()
          });
          return perturbNormal2ArbCall.build(builder, type);
        }
      }
    }]);

    return NormalMapNode;
  }(_TempNode2.default);

  var _default = NormalMapNode;
  _exports.default = _default;
});
