(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/NodeFunction.js", "../core/NodeFunctionInput.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/NodeFunction.js"), require("../core/NodeFunctionInput.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.NodeFunction, global.NodeFunctionInput);
    global.GLSLNodeFunction = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _NodeFunction2, _NodeFunctionInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  _NodeFunction2 = _interopRequireDefault(_NodeFunction2);
  _NodeFunctionInput = _interopRequireDefault(_NodeFunctionInput);

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var declarationRegexp = /^\s*(highp|mediump|lowp)?\s*([a-z_0-9]+)\s*([a-z_0-9]+)?\s*\(([\s\S]*?)\)/i;
  var propertiesRegexp = /[a-z_0-9]+/ig;
  var pragmaMain = '#pragma main';

  var parse = function parse(source) {
    source = source.trim();
    var pragmaMainIndex = source.indexOf(pragmaMain);
    var mainCode = pragmaMainIndex !== -1 ? source.substr(pragmaMainIndex + pragmaMain.length) : source;
    var declaration = mainCode.match(declarationRegexp);

    if (declaration !== null && declaration.length === 5) {
      // tokenizer
      var inputsCode = declaration[4];
      var propsMatches = [];
      var nameMatch = null;

      while ((nameMatch = propertiesRegexp.exec(inputsCode)) !== null) {
        propsMatches.push(nameMatch);
      } // parser


      var inputs = [];
      var i = 0;

      while (i < propsMatches.length) {
        var isConst = propsMatches[i][0] === 'const';

        if (isConst === true) {
          i++;
        }

        var qualifier = propsMatches[i][0];

        if (qualifier === 'in' || qualifier === 'out' || qualifier === 'inout') {
          i++;
        } else {
          qualifier = '';
        }

        var _type = propsMatches[i++][0];
        var count = Number.parseInt(propsMatches[i][0]);
        if (Number.isNaN(count) === false) i++;else count = null;
        var _name = propsMatches[i++][0];
        inputs.push(new _NodeFunctionInput.default(_type, _name, count, qualifier, isConst));
      } //


      var blockCode = mainCode.substring(declaration[0].length);
      var name = declaration[3] !== undefined ? declaration[3] : '';
      var type = declaration[2];
      var presicion = declaration[1] !== undefined ? declaration[1] : '';
      var headerCode = pragmaMainIndex !== -1 ? source.substr(0, pragmaMainIndex) : '';
      return {
        type: type,
        inputs: inputs,
        name: name,
        presicion: presicion,
        inputsCode: inputsCode,
        blockCode: blockCode,
        headerCode: headerCode
      };
    } else {
      throw new Error('FunctionNode: Function is not a GLSL code.');
    }
  };

  var GLSLNodeFunction = /*#__PURE__*/function (_NodeFunction) {
    _inherits(GLSLNodeFunction, _NodeFunction);

    var _super = _createSuper(GLSLNodeFunction);

    function GLSLNodeFunction(source) {
      var _this;

      _classCallCheck(this, GLSLNodeFunction);

      var _parse = parse(source),
          type = _parse.type,
          inputs = _parse.inputs,
          name = _parse.name,
          presicion = _parse.presicion,
          inputsCode = _parse.inputsCode,
          blockCode = _parse.blockCode,
          headerCode = _parse.headerCode;

      _this = _super.call(this, type, inputs, name, presicion);
      _this.inputsCode = inputsCode;
      _this.blockCode = blockCode;
      _this.headerCode = headerCode;
      return _this;
    }

    _createClass(GLSLNodeFunction, [{
      key: "getCode",
      value: function getCode() {
        var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.name;
        var headerCode = this.headerCode;
        var presicion = this.presicion;
        var declarationCode = "".concat(this.type, " ").concat(name, " ( ").concat(this.inputsCode.trim(), " )");

        if (presicion !== '') {
          declarationCode = "".concat(presicion, " ").concat(declarationCode);
        }

        return headerCode + declarationCode + this.blockCode;
      }
    }]);

    return GLSLNodeFunction;
  }(_NodeFunction2.default);

  var _default = GLSLNodeFunction;
  _exports.default = _default;
});
