(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../core/NodeFunction.js", "../core/NodeFunctionInput.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../core/NodeFunction.js"), require("../core/NodeFunctionInput.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.NodeFunction, global.NodeFunctionInput);
    global.WGSLNodeFunction = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _NodeFunction2, _NodeFunctionInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  _NodeFunction2 = _interopRequireDefault(_NodeFunction2);
  _NodeFunctionInput = _interopRequireDefault(_NodeFunctionInput);

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var declarationRegexp = /^fn\s*([a-z_0-9]+)?\s*\(([\s\S]*?)\)\s*\-\>\s*([a-z_0-9]+)?/i;
  var propertiesRegexp = /[a-z_0-9]+/ig;

  var parse = function parse(source) {
    source = source.trim();
    var declaration = source.match(declarationRegexp);

    if (declaration !== null && declaration.length === 4) {
      // tokenizer
      var inputsCode = declaration[2];
      var propsMatches = [];
      var nameMatch = null;

      while ((nameMatch = propertiesRegexp.exec(inputsCode)) !== null) {
        propsMatches.push(nameMatch);
      } // parser


      var inputs = [];
      var i = 0;

      while (i < propsMatches.length) {
        var _name = propsMatches[i++][0];
        var _type = propsMatches[i++][0];
        propsMatches[i++][0]; // precision

        inputs.push(new _NodeFunctionInput.default(_type, _name));
      } //


      var blockCode = source.substring(declaration[0].length);
      var name = declaration[1] !== undefined ? declaration[1] : '';
      var type = declaration[3];
      return {
        type: type,
        inputs: inputs,
        name: name,
        inputsCode: inputsCode,
        blockCode: blockCode
      };
    } else {
      throw new Error('FunctionNode: Function is not a WGSL code.');
    }
  };

  var WGSLNodeFunction = /*#__PURE__*/function (_NodeFunction) {
    _inherits(WGSLNodeFunction, _NodeFunction);

    var _super = _createSuper(WGSLNodeFunction);

    function WGSLNodeFunction(source) {
      var _this;

      _classCallCheck(this, WGSLNodeFunction);

      var _parse = parse(source),
          type = _parse.type,
          inputs = _parse.inputs,
          name = _parse.name,
          inputsCode = _parse.inputsCode,
          blockCode = _parse.blockCode;

      _this = _super.call(this, type, inputs, name);
      _this.inputsCode = inputsCode;
      _this.blockCode = blockCode;
      return _this;
    }

    _createClass(WGSLNodeFunction, [{
      key: "getCode",
      value: function getCode() {
        var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.name;
        return "fn ".concat(name, " ( ").concat(this.inputsCode.trim(), " ) -> ").concat(this.type) + this.blockCode;
      }
    }]);

    return WGSLNodeFunction;
  }(_NodeFunction2.default);

  var _default = WGSLNodeFunction;
  _exports.default = _default;
});
