(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../libs/stats.module.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../libs/stats.module.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.statsModule);
    global.GPUStatsPanel = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _statsModule) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.GPUStatsPanel = void 0;
  _statsModule = _interopRequireDefault(_statsModule);

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  // https://www.khronos.org/registry/webgl/extensions/EXT_disjoint_timer_query/
  // https://www.khronos.org/registry/webgl/extensions/EXT_disjoint_timer_query_webgl2/
  var GPUStatsPanel = /*#__PURE__*/function (_Stats$Panel) {
    _inherits(GPUStatsPanel, _Stats$Panel);

    var _super = _createSuper(GPUStatsPanel);

    function GPUStatsPanel(context) {
      var _this;

      var name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'GPU MS';

      _classCallCheck(this, GPUStatsPanel);

      _this = _super.call(this, name, '#f90', '#210');
      var isWebGL2 = true;
      var extension = context.getExtension('EXT_disjoint_timer_query_webgl2');

      if (extension === null) {
        isWebGL2 = false;
        extension = context.getExtension('EXT_disjoint_timer_query');

        if (extension === null) {
          console.warn('GPUStatsPanel: disjoint_time_query extension not available.');
        }
      }

      _this.context = context;
      _this.extension = extension;
      _this.maxTime = 30;
      _this.activeQueries = 0;

      _this.startQuery = function () {
        var _this2 = this;

        var gl = this.context;
        var ext = this.extension;

        if (ext === null) {
          return;
        } // create the query object


        var query;

        if (isWebGL2) {
          query = gl.createQuery();
          gl.beginQuery(ext.TIME_ELAPSED_EXT, query);
        } else {
          query = ext.createQueryEXT();
          ext.beginQueryEXT(ext.TIME_ELAPSED_EXT, query);
        }

        this.activeQueries++;

        var checkQuery = function checkQuery() {
          // check if the query is available and valid
          var available, disjoint, ns;

          if (isWebGL2) {
            available = gl.getQueryParameter(query, gl.QUERY_RESULT_AVAILABLE);
            disjoint = gl.getParameter(ext.GPU_DISJOINT_EXT);
            ns = gl.getQueryParameter(query, gl.QUERY_RESULT);
          } else {
            available = ext.getQueryObjectEXT(query, ext.QUERY_RESULT_AVAILABLE_EXT);
            disjoint = gl.getParameter(ext.GPU_DISJOINT_EXT);
            ns = ext.getQueryObjectEXT(query, ext.QUERY_RESULT_EXT);
          }

          var ms = ns * 1e-6;

          if (available) {
            // update the display if it is valid
            if (!disjoint) {
              _this2.update(ms, _this2.maxTime);
            }

            _this2.activeQueries--;
          } else {
            // otherwise try again the next frame
            requestAnimationFrame(checkQuery);
          }
        };

        requestAnimationFrame(checkQuery);
      };

      _this.endQuery = function () {
        // finish the query measurement
        var ext = this.extension;
        var gl = this.context;

        if (ext === null) {
          return;
        }

        if (isWebGL2) {
          gl.endQuery(ext.TIME_ELAPSED_EXT);
        } else {
          ext.endQueryEXT(ext.TIME_ELAPSED_EXT);
        }
      };

      return _this;
    }

    return _createClass(GPUStatsPanel);
  }(_statsModule.default.Panel);

  _exports.GPUStatsPanel = GPUStatsPanel;
});
