(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three);
    global.SceneUtils = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.attach = attach;
  _exports.createMeshesFromInstancedMesh = createMeshesFromInstancedMesh;
  _exports.createMultiMaterialObject = createMultiMaterialObject;
  _exports.detach = detach;

  function createMeshesFromInstancedMesh(instancedMesh) {
    var group = new _three.Group();
    var count = instancedMesh.count;
    var geometry = instancedMesh.geometry;
    var material = instancedMesh.material;

    for (var i = 0; i < count; i++) {
      var mesh = new _three.Mesh(geometry, material);
      instancedMesh.getMatrixAt(i, mesh.matrix);
      mesh.matrix.decompose(mesh.position, mesh.quaternion, mesh.scale);
      group.add(mesh);
    }

    group.copy(instancedMesh);
    group.updateMatrixWorld(); // ensure correct world matrices of meshes

    return group;
  }

  function createMultiMaterialObject(geometry, materials) {
    var group = new _three.Group();

    for (var i = 0, l = materials.length; i < l; i++) {
      group.add(new _three.Mesh(geometry, materials[i]));
    }

    return group;
  }

  function detach(child, parent, scene) {
    console.warn('THREE.SceneUtils: detach() has been deprecated. Use scene.attach( child ) instead.');
    scene.attach(child);
  }

  function attach(child, scene, parent) {
    console.warn('THREE.SceneUtils: attach() has been deprecated. Use parent.attach( child ) instead.');
    parent.attach(child);
  }
});
