(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "./XRHandMeshModel.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("./XRHandMeshModel.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.XRHandMeshModel);
    global.OculusHandModel = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, _XRHandMeshModel) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.OculusHandModel = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _get() { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(arguments.length < 3 ? target : receiver); } return desc.value; }; } return _get.apply(this, arguments); }

  function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var TOUCH_RADIUS = 0.01;
  var POINTING_JOINT = 'index-finger-tip';

  var OculusHandModel = /*#__PURE__*/function (_Object3D) {
    _inherits(OculusHandModel, _Object3D);

    var _super = _createSuper(OculusHandModel);

    function OculusHandModel(controller) {
      var _this;

      _classCallCheck(this, OculusHandModel);

      _this = _super.call(this);
      _this.controller = controller;
      _this.motionController = null;
      _this.envMap = null;
      _this.mesh = null;
      controller.addEventListener('connected', function (event) {
        var xrInputSource = event.data;

        if (xrInputSource.hand && !_this.motionController) {
          _this.xrInputSource = xrInputSource;
          _this.motionController = new _XRHandMeshModel.XRHandMeshModel(_assertThisInitialized(_this), controller, _this.path, xrInputSource.handedness);
        }
      });
      controller.addEventListener('disconnected', function () {
        _this.clear();

        _this.motionController = null;
      });
      return _this;
    }

    _createClass(OculusHandModel, [{
      key: "updateMatrixWorld",
      value: function updateMatrixWorld(force) {
        _get(_getPrototypeOf(OculusHandModel.prototype), "updateMatrixWorld", this).call(this, force);

        if (this.motionController) {
          this.motionController.updateMesh();
        }
      }
    }, {
      key: "getPointerPosition",
      value: function getPointerPosition() {
        var indexFingerTip = this.controller.joints[POINTING_JOINT];

        if (indexFingerTip) {
          return indexFingerTip.position;
        } else {
          return null;
        }
      }
    }, {
      key: "intersectBoxObject",
      value: function intersectBoxObject(boxObject) {
        var pointerPosition = this.getPointerPosition();

        if (pointerPosition) {
          var indexSphere = new _three.Sphere(pointerPosition, TOUCH_RADIUS);
          var box = new _three.Box3().setFromObject(boxObject);
          return indexSphere.intersectsBox(box);
        } else {
          return false;
        }
      }
    }, {
      key: "checkButton",
      value: function checkButton(button) {
        if (this.intersectBoxObject(button)) {
          button.onPress();
        } else {
          button.onClear();
        }

        if (button.isPressed()) {
          button.whilePressed();
        }
      }
    }]);

    return OculusHandModel;
  }(_three.Object3D);

  _exports.OculusHandModel = OculusHandModel;
});
