(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "three", "../loaders/GLTFLoader.js", "../libs/motion-controllers.module.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("three"), require("../loaders/GLTFLoader.js"), require("../libs/motion-controllers.module.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.three, global.GLTFLoader, global.motionControllersModule);
    global.XRControllerModelFactory = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _three, _GLTFLoader, _motionControllersModule) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.XRControllerModelFactory = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  function _get() { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(arguments.length < 3 ? target : receiver); } return desc.value; }; } return _get.apply(this, arguments); }

  function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } Object.defineProperty(subClass, "prototype", { value: Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }), writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

  function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  var DEFAULT_PROFILES_PATH = 'https://cdn.jsdelivr.net/npm/@webxr-input-profiles/assets@1.0/dist/profiles';
  var DEFAULT_PROFILE = 'generic-trigger';

  var XRControllerModel = /*#__PURE__*/function (_Object3D) {
    _inherits(XRControllerModel, _Object3D);

    var _super = _createSuper(XRControllerModel);

    function XRControllerModel() {
      var _this;

      _classCallCheck(this, XRControllerModel);

      _this = _super.call(this);
      _this.motionController = null;
      _this.envMap = null;
      return _this;
    }

    _createClass(XRControllerModel, [{
      key: "setEnvironmentMap",
      value: function setEnvironmentMap(envMap) {
        var _this2 = this;

        if (this.envMap == envMap) {
          return this;
        }

        this.envMap = envMap;
        this.traverse(function (child) {
          if (child.isMesh) {
            child.material.envMap = _this2.envMap;
            child.material.needsUpdate = true;
          }
        });
        return this;
      }
      /**
       * Polls data from the XRInputSource and updates the model's components to match
       * the real world data
       */

    }, {
      key: "updateMatrixWorld",
      value: function updateMatrixWorld(force) {
        _get(_getPrototypeOf(XRControllerModel.prototype), "updateMatrixWorld", this).call(this, force);

        if (!this.motionController) return; // Cause the MotionController to poll the Gamepad for data

        this.motionController.updateFromGamepad(); // Update the 3D model to reflect the button, thumbstick, and touchpad state

        Object.values(this.motionController.components).forEach(function (component) {
          // Update node data based on the visual responses' current states
          Object.values(component.visualResponses).forEach(function (visualResponse) {
            var valueNode = visualResponse.valueNode,
                minNode = visualResponse.minNode,
                maxNode = visualResponse.maxNode,
                value = visualResponse.value,
                valueNodeProperty = visualResponse.valueNodeProperty; // Skip if the visual response node is not found. No error is needed,
            // because it will have been reported at load time.

            if (!valueNode) return; // Calculate the new properties based on the weight supplied

            if (valueNodeProperty === _motionControllersModule.Constants.VisualResponseProperty.VISIBILITY) {
              valueNode.visible = value;
            } else if (valueNodeProperty === _motionControllersModule.Constants.VisualResponseProperty.TRANSFORM) {
              valueNode.quaternion.slerpQuaternions(minNode.quaternion, maxNode.quaternion, value);
              valueNode.position.lerpVectors(minNode.position, maxNode.position, value);
            }
          });
        });
      }
    }]);

    return XRControllerModel;
  }(_three.Object3D);
  /**
   * Walks the model's tree to find the nodes needed to animate the components and
   * saves them to the motionContoller components for use in the frame loop. When
   * touchpads are found, attaches a touch dot to them.
   */


  function findNodes(motionController, scene) {
    // Loop through the components and find the nodes needed for each components' visual responses
    Object.values(motionController.components).forEach(function (component) {
      var type = component.type,
          touchPointNodeName = component.touchPointNodeName,
          visualResponses = component.visualResponses;

      if (type === _motionControllersModule.Constants.ComponentType.TOUCHPAD) {
        component.touchPointNode = scene.getObjectByName(touchPointNodeName);

        if (component.touchPointNode) {
          // Attach a touch dot to the touchpad.
          var sphereGeometry = new _three.SphereGeometry(0.001);
          var material = new _three.MeshBasicMaterial({
            color: 0x0000FF
          });
          var sphere = new _three.Mesh(sphereGeometry, material);
          component.touchPointNode.add(sphere);
        } else {
          console.warn("Could not find touch dot, ".concat(component.touchPointNodeName, ", in touchpad component ").concat(component.id));
        }
      } // Loop through all the visual responses to be applied to this component


      Object.values(visualResponses).forEach(function (visualResponse) {
        var valueNodeName = visualResponse.valueNodeName,
            minNodeName = visualResponse.minNodeName,
            maxNodeName = visualResponse.maxNodeName,
            valueNodeProperty = visualResponse.valueNodeProperty; // If animating a transform, find the two nodes to be interpolated between.

        if (valueNodeProperty === _motionControllersModule.Constants.VisualResponseProperty.TRANSFORM) {
          visualResponse.minNode = scene.getObjectByName(minNodeName);
          visualResponse.maxNode = scene.getObjectByName(maxNodeName); // If the extents cannot be found, skip this animation

          if (!visualResponse.minNode) {
            console.warn("Could not find ".concat(minNodeName, " in the model"));
            return;
          }

          if (!visualResponse.maxNode) {
            console.warn("Could not find ".concat(maxNodeName, " in the model"));
            return;
          }
        } // If the target node cannot be found, skip this animation


        visualResponse.valueNode = scene.getObjectByName(valueNodeName);

        if (!visualResponse.valueNode) {
          console.warn("Could not find ".concat(valueNodeName, " in the model"));
        }
      });
    });
  }

  function addAssetSceneToControllerModel(controllerModel, scene) {
    // Find the nodes needed for animation and cache them on the motionController.
    findNodes(controllerModel.motionController, scene); // Apply any environment map that the mesh already has set.

    if (controllerModel.envMap) {
      scene.traverse(function (child) {
        if (child.isMesh) {
          child.material.envMap = controllerModel.envMap;
          child.material.needsUpdate = true;
        }
      });
    } // Add the glTF scene to the controllerModel.


    controllerModel.add(scene);
  }

  var XRControllerModelFactory = /*#__PURE__*/function () {
    function XRControllerModelFactory() {
      var gltfLoader = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

      _classCallCheck(this, XRControllerModelFactory);

      this.gltfLoader = gltfLoader;
      this.path = DEFAULT_PROFILES_PATH;
      this._assetCache = {}; // If a GLTFLoader wasn't supplied to the constructor create a new one.

      if (!this.gltfLoader) {
        this.gltfLoader = new _GLTFLoader.GLTFLoader();
      }
    }

    _createClass(XRControllerModelFactory, [{
      key: "createControllerModel",
      value: function createControllerModel(controller) {
        var _this3 = this;

        var controllerModel = new XRControllerModel();
        var scene = null;
        controller.addEventListener('connected', function (event) {
          var xrInputSource = event.data;
          if (xrInputSource.targetRayMode !== 'tracked-pointer' || !xrInputSource.gamepad) return;
          (0, _motionControllersModule.fetchProfile)(xrInputSource, _this3.path, DEFAULT_PROFILE).then(function (_ref) {
            var profile = _ref.profile,
                assetPath = _ref.assetPath;
            controllerModel.motionController = new _motionControllersModule.MotionController(xrInputSource, profile, assetPath);
            var cachedAsset = _this3._assetCache[controllerModel.motionController.assetUrl];

            if (cachedAsset) {
              scene = cachedAsset.scene.clone();
              addAssetSceneToControllerModel(controllerModel, scene);
            } else {
              if (!_this3.gltfLoader) {
                throw new Error('GLTFLoader not set.');
              }

              _this3.gltfLoader.setPath('');

              _this3.gltfLoader.load(controllerModel.motionController.assetUrl, function (asset) {
                _this3._assetCache[controllerModel.motionController.assetUrl] = asset;
                scene = asset.scene.clone();
                addAssetSceneToControllerModel(controllerModel, scene);
              }, null, function () {
                throw new Error("Asset ".concat(controllerModel.motionController.assetUrl, " missing or malformed."));
              });
            }
          }).catch(function (err) {
            console.warn(err);
          });
        });
        controller.addEventListener('disconnected', function () {
          controllerModel.motionController = null;
          controllerModel.remove(scene);
          scene = null;
        });
        return controllerModel;
      }
    }]);

    return XRControllerModelFactory;
  }();

  _exports.XRControllerModelFactory = XRControllerModelFactory;
});
