(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "../loaders/GLTFLoader.js"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("../loaders/GLTFLoader.js"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.GLTFLoader);
    global.XRHandMeshModel = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _GLTFLoader) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.XRHandMeshModel = void 0;

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

  var DEFAULT_HAND_PROFILE_PATH = 'https://cdn.jsdelivr.net/npm/@webxr-input-profiles/assets@1.0/dist/profiles/generic-hand/';

  var XRHandMeshModel = /*#__PURE__*/function () {
    function XRHandMeshModel(handModel, controller, path, handedness) {
      var _this = this;

      _classCallCheck(this, XRHandMeshModel);

      this.controller = controller;
      this.handModel = handModel;
      this.bones = [];
      var loader = new _GLTFLoader.GLTFLoader();
      loader.setPath(path || DEFAULT_HAND_PROFILE_PATH);
      loader.load("".concat(handedness, ".glb"), function (gltf) {
        var object = gltf.scene.children[0];

        _this.handModel.add(object);

        var mesh = object.getObjectByProperty('type', 'SkinnedMesh');
        mesh.frustumCulled = false;
        mesh.castShadow = true;
        mesh.receiveShadow = true;
        var joints = ['wrist', 'thumb-metacarpal', 'thumb-phalanx-proximal', 'thumb-phalanx-distal', 'thumb-tip', 'index-finger-metacarpal', 'index-finger-phalanx-proximal', 'index-finger-phalanx-intermediate', 'index-finger-phalanx-distal', 'index-finger-tip', 'middle-finger-metacarpal', 'middle-finger-phalanx-proximal', 'middle-finger-phalanx-intermediate', 'middle-finger-phalanx-distal', 'middle-finger-tip', 'ring-finger-metacarpal', 'ring-finger-phalanx-proximal', 'ring-finger-phalanx-intermediate', 'ring-finger-phalanx-distal', 'ring-finger-tip', 'pinky-finger-metacarpal', 'pinky-finger-phalanx-proximal', 'pinky-finger-phalanx-intermediate', 'pinky-finger-phalanx-distal', 'pinky-finger-tip'];
        joints.forEach(function (jointName) {
          var bone = object.getObjectByName(jointName);

          if (bone !== undefined) {
            bone.jointName = jointName;
          } else {
            console.warn("Couldn't find ".concat(jointName, " in ").concat(handedness, " hand mesh"));
          }

          _this.bones.push(bone);
        });
      });
    }

    _createClass(XRHandMeshModel, [{
      key: "updateMesh",
      value: function updateMesh() {
        // XR Joints
        var XRJoints = this.controller.joints;

        for (var i = 0; i < this.bones.length; i++) {
          var bone = this.bones[i];

          if (bone) {
            var XRJoint = XRJoints[bone.jointName];

            if (XRJoint.visible) {
              var position = XRJoint.position;

              if (bone) {
                bone.position.copy(position);
                bone.quaternion.copy(XRJoint.quaternion); // bone.scale.setScalar( XRJoint.jointRadius || defaultRadius );
              }
            }
          }
        }
      }
    }]);

    return XRHandMeshModel;
  }();

  _exports.XRHandMeshModel = XRHandMeshModel;
});
